/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

import java.awt.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.BitSet;

import org.jet.tracks.*;

/**
 * This class defines basic behaviour for scenery editor.
 *
 * @author Adam Domaski
 * @version 0.1 10/03/2004
 */
public class BasicSceneryEditorUI extends SceneryEditorUI
	implements SceneryConstants
{
	/** Class text represenataion. */
	public static final String UI_CLASS_ID = "SceneryEditorUI";

	/**
	 * Normal mouse work mode.
	 * In this mode mouse operates to selected for track.
	 */
	public static final int NORMAL_MODE = 0;

	/**
	 * Zoom mouse work mode.
	 * In this mode mouse operates to zoom in / zoom out scenery. It
	 * deos not select tracks.
	 */
	public static final int ZOOM_MODE = 1;

	private static final Vector3d VECTOR_ZERO = new Vector3d();

	/*
	 * ------------------ Constructors ----------------
	 */
	
	/**
	 * Default constructor.
	 */
	public BasicSceneryEditorUI()
	{
	}

	/*
	 * -------------- UI ----------------
	 */

	/**
	 * Creates component UI for the component.
	 *
	 * @param c Component to create user interface object for.
	 * @return New <code>BasicSceneryEditorUI</code>.
	 */
	public static ComponentUI createUI(JComponent c)
	{
		return new BasicSceneryEditorUI();
	}

	/*
	 * ----------------- Installation ---------------
	 */

	/**
	 * Installs defaults, listeners for component.
	 *
	 * @param c A component.
	 */
	public void installUI(JComponent c)
	{
		sce = (JSceneryEditor) c;

		prepareToInstall();		
		installDefaults();
		installListeners();
		installKeyboardActions();
	}

	/**
	 * First stage of component installation.
	 */
	protected void prepareToInstall()
	{
		scenery = sce.getScenery();
		selectionModel = new MemorableSelectionModel(sce);
		sce.setSelectionModel(selectionModel);

		cachedTracks = new ArrayList();
		cachedSwitches = new ArrayList();
		cachedTrackNames = new ArrayList();
	}

	/**
	 * Installs defaults for the component (e.g. colors).
	 */
	protected void installDefaults()
	{
		if (sce.getBackground() == null
			|| sce.getBackground() instanceof UIResource)
		{
			sce.setBackground(UIManager.getColor("ToggleButton.select"));
		}
		sce.setGridColor(UIManager.getColor("ComboBox.background"));
		sce.setOpaque(true);
		sce.setDoubleBuffered(true);
		sce.setFocusable(true);

		installCursors();
		installImages();

		setTrackColor(Color.blue);
		setSwitchColor(Color.green);
		setTrackNamesColor( new Color(255, 235, 52) );
		setTrackNamesVisibility(true);

		preferredSize = new Dimension();

		mouseMode = NORMAL_MODE;
		oldCursor = sce.getCursor();
	}

	/**
	 * Installs zoom cursor.
	 */
	protected void installCursors()
	{
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Image image = toolkit.getImage("images/cur_zoom.png");
		zoomCursor = toolkit.createCustomCursor(
			image, new Point(0, 0), "zoomCursor");
	}

	/**
	 * Installs images.
	 */
	protected void installImages()
	{
		imageRedHandler = new ImageIcon("images/se_handler_red.png").getImage();
		imageBasePoint = new ImageIcon("images/se_handler_yellow.png").getImage();
	}

	/**
	 * Installs listeners for the component.
	 */
	protected void installListeners()
	{
		if ((sceneryListener = createSceneryListener()) != null
			&& scenery != null)
			scenery.addSceneryChangeListener(sceneryListener);
		if ((highlighterListener = createHighlighterListener()) != null
			&& sce != null)
		{
			sce.addMouseListener(highlighterListener);
		}
		if ((propertyChangeListener = createPropertyChangeListener()) != null
			&& sce != null)
		{
			sce.addPropertyChangeListener(propertyChangeListener);
		}
		if ((keyListener = createKeyListener()) != null
			&& sce != null)
		{
			sce.addKeyListener(keyListener);
		}
		if ((scenerySelectionListener = createScenerySelectionListener()) != null &&
			selectionModel != null)
		{
			selectionModel.addScenerySelectionListener(scenerySelectionListener);
		}
	}

	/**
	 * Creates property handler to handle property changes in
	 * the component.
	 */
	protected PropertyChangeHandler createPropertyChangeListener()
	{
		return new PropertyChangeHandler();
	}

	/**
	 * Creates scenery listener to handle changes in
	 * the scenery.
	 */
	protected SceneryChangeHandler createSceneryListener()
	{
		return new SceneryChangeHandler();
	}

	/**
	 * Creates object highlight listener.
	 */
	protected HighlighterHandler createHighlighterListener()
	{
		return new HighlighterHandler();
	}

	/**
	 * Creates key listener to listen to key.
	 */
	protected KeyHandler createKeyListener()
	{
		return new KeyHandler();
	}

	/**
	 * Creates scenery selection listener.
	 */
	protected SelectionHandler createScenerySelectionListener()
	{
		return new SelectionHandler();
	}

	/**
	 * Installs keyboard action for the scenery editor.
	 */
	protected void installKeyboardActions()
	{
		ActionMap aMap = sce.getActionMap();

		aMap.put("zoom in", new ZoomInAction(sce));
		aMap.put("zoom out", new ZoomOutAction(sce));

		InputMap iMap = sce.getInputMap();

		iMap.put(KeyStroke.getKeyStroke('+'), "zoom in");
		iMap.put(KeyStroke.getKeyStroke('-'), "zoom out");
	}

	/*
	 * ------------- Uninstalling --------------
	 */

	/**
	 * Uninstalls user interface for a component.
	 *
	 * @param c Component to uninstall (the same as in install).
	 */
	public void uninstallUI(JComponent c)
	{
		uninstallKeyboardActions();
		uninstallListeners();

		completeUninstall();
	}

	/**
	 * Last stage of uninstalling process.
	 * It nullize all members.
	 */
	protected void completeUninstall()
	{
		scenery = null;
		sceneryListener = null;
		highlighterListener = null;
		propertyChangeListener = null;
		keyListener = null;
		scenerySelectionListener = null;
		preferredSize = null;
		zoomCursor = null;
		sce = null;
		selectionModel = null;

		cachedTracks = null;
		cachedSwitches = null;
		cachedTrackNames = null;
	}

	/**
	 * Uninstalls listeners.
	 */
	protected void uninstallListeners()
	{
		if (scenery != null && sceneryListener != null)
			scenery.removeSceneryChangeListener(sceneryListener);
		if (highlighterListener != null)
			sce.removeMouseListener(highlighterListener);
		if (propertyChangeListener != null)
			sce.removePropertyChangeListener(propertyChangeListener);
		if (keyListener != null)
			sce.removeKeyListener(keyListener);
		if (scenerySelectionListener != null)
			selectionModel.removeScenerySelectionListener(scenerySelectionListener);
	}

	/**
	 * Uninstalls keyboard actions.
	 */
	protected void uninstallKeyboardActions()
	{
		sce.getInputMap().clear();
		sce.getActionMap().clear();
	}

	/*
	 * -------------- Sceneries -----------------
	 */
	
	/**
	 * Sets the model for this scenery editor user interface.
	 *
	 * @param s The new scenery.
	 */
	protected void setModel(Scenery s)
	{
		if (scenery != null && sceneryListener != null)
			scenery.removeSceneryChangeListener(sceneryListener);
		scenery = s;
		if (scenery != null && sceneryListener != null)
			scenery.addSceneryChangeListener(sceneryListener);
		if (sce != null)
		{
			validPreferredSize = false;
			validCachedDisplay = false;
			validMinMaxInfo = false;
			updatePreferredSize();
			sce.revalidate();
			sce.repaint();

			// Scroll rect to visible
			SwingUtilities.invokeLater(new Runnable() {
				public void run()
				{
					Point p = new Point();
					p.x = preferredSize.width / 2;
					p.y = preferredSize.height / 2;
					sce.centerViewAt(p);
				}
			});
		}
	}

	/**
	 * Resets the selection model. The appropriate listeners are installed
	 * on the model.
	 */
	protected void setSelectionModel(ScenerySelectionModel newSSM)
	{
		if (scenerySelectionListener != null && selectionModel != null)
			selectionModel.removeScenerySelectionListener(
				scenerySelectionListener);
		selectionModel = newSSM;
		if (scenerySelectionListener != null && selectionModel != null)
			selectionModel.addScenerySelectionListener(
				scenerySelectionListener);
		if (sce != null)
			sce.repaint();
	}

	/*
	 * ------------- Mouse mode -----------------
	 */
	
	/**
	 * Returns mouse work mode.
	 *
	 * @return Code of mouse mode.
	 * @see #NORMAL_MODE
	 * @see #ZOOM_MODE
	 */
	public int getMouseMode()
	{
		return mouseMode;
	}

	/**
	 * Sets mouse work mode.
	 *
	 * @param mode Code of mouse mode.
	 */
	public void setMouseMode(int mode)
	{
		if (mode != NORMAL_MODE && mode != ZOOM_MODE)
			throw new IllegalArgumentException();

		mouseMode = mode;
		if (mode == NORMAL_MODE)
		{
			sce.setCursor(oldCursor);
		}
		else if (mode == ZOOM_MODE)
		{
			oldCursor = sce.getCursor();
			sce.setCursor(zoomCursor);
		}
	}

	/*
	 * --------------- Painting -----------------
	 */

	/**
	 * Paints a component <code>c</code> with graphic context <code>g</code>.
	 *
	 * @param g Graphic context.
	 * @param c The component to paint.
	 */
	public void paint(Graphics g, JComponent c)
	{
		if (c != sce)
			throw new InternalError("SceneryEditor can't paint");

		//long time = System.currentTimeMillis();

		paintBackground(g);
		paintForeground(g);

		//long aTime = System.currentTimeMillis();
		//aTime = aTime - time;

		//System.out.println("Drawn in " + aTime);
	}

	/**
	 * Paints background for component.
	 * This method is invoken from <code>paint</code> function.
	 *
	 * @param g Graphic context.
	 */
	protected void paintBackground(Graphics g)
	{
		if (sce.isGridEnabled())
			paintGrid(g);
	}

	/**
	 * Paints a grid for scenery editor.
	 *
	 * @param g Graphic context.
	 */
	protected void paintGrid(Graphics g)
	{
		Rectangle rr = (Rectangle) sce.getBounds();
		g.setColor( sce.getGridColor() );

		double scaller = sce.getScale() * PIXELS_PER_METER;
		int meters = ((int) (1 / sce.getScale())) * 5;
		double dStartX = ((int) (maxX / meters)) * meters;
		double dStartY = ((int) (maxY / meters)) * meters;

		double startX = modelToView(maxX - dStartX);
		if (modelToView(maxX - minX) > getMinimumSize(sce).width - 2 * PIXELS_PER_METER)
		{
			startX += PIXELS_PER_METER;
		}
		else
		{
			startX += (getMinimumSize(sce).width - modelToView(maxX - minX)) / 2;
		}

		double startY = modelToView(maxY - dStartY);
		if (modelToView(maxY - minY) > getMinimumSize(sce).height - 2 * PIXELS_PER_METER)
		{
			startY += PIXELS_PER_METER;
		}
		else
		{
			startY += (getMinimumSize(sce).height - modelToView(maxY - minY)) / 2;
		}

		while (startX > 0)
		{
			startX -= meters * scaller;
		}
		if (startX <= 0)
		{
			startX += meters * scaller;
		}

		while (startY > 0)
		{
			startY -= meters * scaller;
		}
		if (startY <= 0)
		{
			startY += meters * scaller;
		}

		while (startX < rr.width)
		{
			g.drawLine((int) startX, 0, (int) startX, rr.height);
			double oldX = dStartX;
			dStartX = dStartX + meters;
			startX += modelToView(dStartX - oldX);			
		}

		while (startY < rr.height)
		{
			g.drawLine(0, (int) startY, rr.width, (int) startY);
			double oldY = dStartY;
			dStartY = dStartY - meters;
			startY += modelToView(oldY - dStartY);
		}
	}

	/**
	 * Draws tracks.
	 * <h3>This method is not good. It isn't optimised at now.</h3>
	 */
	protected void paintForeground(Graphics g)
	{
		if (g instanceof Graphics2D)
		{
			Graphics2D g2 = (Graphics2D) g;

			if (!validCachedDisplay)
			{
				updateCachedDisplay();
			}

			paintTracks(g2);
			paintSwitches(g2);
			paintTrackNames(g2);
			paintSelection(g2);
			paintHandles(g2);
		}
	}

	/**
	 * Paints tracks.
	 * Separated for readability reason.
	 */
	protected void paintTracks(Graphics2D g2)
	{
		Stroke oldStroke = null;
		Rectangle rect = g2.getClipBounds();
		g2.setColor( getTrackColor() );

		double scl = sce.getScale();

		if (scl == 1.0)
		{
			oldStroke = g2.getStroke();
			g2.setStroke(new BasicStroke(1.435f * PIXELS_PER_METER,
				BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
		}
		else if (scl >= 0.1 && scl < 1.0)
		{
			oldStroke = g2.getStroke();
			g2.setStroke(new BasicStroke(2f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
		}

		Iterator iterator = cachedTracks.iterator();
		if (scl < 0.001)
		{
			while (iterator.hasNext())
			{
				CachedTrack cachedTrack = (CachedTrack) iterator.next();
				if (shouldDraw(rect, cachedTrack.cc))
				{
					g2.drawLine((int) cachedTrack.cc.getX1(), (int) cachedTrack.cc.getY1(),
						(int) cachedTrack.cc.getX2(), (int) cachedTrack.cc.getY2());
				}
			}
		}
		else
		{
			while (iterator.hasNext())
			{
				CachedTrack cachedTrack = (CachedTrack) iterator.next();
				if (shouldDraw(rect, cachedTrack.cc))
				{
					g2.draw(cachedTrack.cc);
				}
			}
		}

		if (scl >= 0.1)
		{
			g2.setStroke(oldStroke);
		}
	}

	/**
	 * Paints switches.
	 * Separated for readability reason.
	 */
	protected void paintSwitches(Graphics2D g2)
	{
		Stroke oldStroke = null;
		Rectangle rect = g2.getClipBounds();
		g2.setColor( getSwitchColor() );

		double scl = sce.getScale();

		if (scl == 1.0)
		{
			oldStroke = g2.getStroke();
			g2.setStroke(new BasicStroke(1.435f * PIXELS_PER_METER,
				BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
		}
		else if (scl >= 0.1 && scl < 1.0)
		{
			oldStroke = g2.getStroke();
			g2.setStroke(new BasicStroke(2f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
		}

		Iterator iterator = cachedSwitches.iterator();
		if (scl < 0.001)
		{
			while (iterator.hasNext())
			{
				CachedSwitch cachedSwitch = (CachedSwitch) iterator.next();
				if (shouldDraw(rect, cachedSwitch.cc1))
				{
					g2.drawLine((int) cachedSwitch.cc1.getX1(), (int) cachedSwitch.cc1.getY1(),
						(int) cachedSwitch.cc1.getX2(), (int) cachedSwitch.cc1.getY2());
				}
				if (shouldDraw(rect, cachedSwitch.cc2))
				{
					g2.drawLine((int) cachedSwitch.cc2.getX1(), (int) cachedSwitch.cc2.getY1(),
						(int) cachedSwitch.cc2.getX2(), (int) cachedSwitch.cc2.getY2());
				}
			}
		}
		else
		{
			while (iterator.hasNext())
			{
				CachedSwitch cachedSwitch = (CachedSwitch) iterator.next();
				if (shouldDraw(rect, cachedSwitch.cc1))
				{
					g2.draw(cachedSwitch.cc1);
				}
				if (shouldDraw(rect, cachedSwitch.cc2))
				{
					g2.draw(cachedSwitch.cc2);
				}
			}
 		}

		if (scl >= 0.1)
		{
			g2.setStroke(oldStroke);
		}
	}

	/**
	 * Paints track names.
	 */
	protected void paintTrackNames(Graphics2D g2)
	{
		if (!trackNamesVisibility)
			return;

		if (sce.getScale() < 0.05)
			return;

		Rectangle rect = g2.getClipBounds();
		g2.setColor( getTrackNamesColor() );

		Iterator iterator = cachedTrackNames.iterator();
		while (iterator.hasNext())
		{
			CachedTextBox cachedTextBox = (CachedTextBox) iterator.next();

			if (shouldDraw(rect, cachedTextBox.tb.getBounds()))
			{
				cachedTextBox.tb.draw(g2);
			}
		}
	}

	/**
	 * Paints selection.
	 */
	protected void paintSelection(Graphics2D g2)
	{
		Stroke oldStroke = null;
		Rectangle rect = g2.getClipBounds();

		if (!selectionModel.isSelectionEmpty())
		{
			Object o = selectionModel.getSelectedObject();

			double dHeight = imageRedHandler.getHeight(sce) / 2;
			double dWidth = imageRedHandler.getWidth(sce) / 2;

			g2.setColor(Color.RED);
			oldStroke = g2.getStroke();

			double scl = sce.getScale();

			if (scl == 1.0)
			{
				g2.setStroke(new BasicStroke(1.435f * PIXELS_PER_METER,
					BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
			}
			else if (scl >= 0.1 && scl < 1.0)
			{
				g2.setStroke(new BasicStroke(2f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
			}

			if (o instanceof Switch)
			{
				Switch _switch = (Switch) o;
				CubicCurve2D[] cc = transformSwitchToView(_switch);
				if (shouldDraw(rect, cc[0]))
					g2.draw(cc[0]);
				if (shouldDraw(rect, cc[1]))
					g2.draw(cc[1]);
			}
			else if (o instanceof Track)
			{
				Track track = (Track) o;
				CubicCurve2D cc = transformTrackToView(track);
				if (shouldDraw(rect, cc))
					g2.draw(cc);
			}

			g2.setStroke(oldStroke);
		}
	}

	/**
	 * Paints selection handlers if there is selection.
	 * It paints also a base point.
	 */
	protected void paintHandles(Graphics2D g2)
	{
		BasePoint basePoint = sce.getBasePoint();
		Rectangle rect = g2.getClipBounds();

		// Paint selection
		if (!selectionModel.isSelectionEmpty())
		{
			Object o = selectionModel.getSelectedObject();

			double dHeight = imageRedHandler.getHeight(sce) / 2;
			double dWidth = imageRedHandler.getWidth(sce) / 2;

			if (o instanceof Switch)
			{
				Switch sswitch = (Switch) o;
				CubicCurve2D[] cc = transformSwitchToView(sswitch);

				if (basePoint == null || !basePoint.getPoint().equals( sswitch.getPoint1() ))
				{
					g2.drawImage(imageRedHandler, (int) (cc[0].getP1().getX() - dWidth),
						(int) (cc[0].getP1().getY() - dHeight), sce);
				}
				if (basePoint == null || !basePoint.getPoint().equals( sswitch.getPoint2() ))
				{
					g2.drawImage(imageRedHandler, (int) (cc[0].getP2().getX() - dWidth),
						(int) (cc[0].getP2().getY() - dHeight), sce);
				}
				if (basePoint == null || !basePoint.getPoint().equals( sswitch.getPoint3() ))
				{
					g2.drawImage(imageRedHandler, (int) (cc[1].getP2().getX() - dWidth),
						(int) (cc[1].getP2().getY() - dHeight), sce);
				}
			}
			else if (o instanceof Track)
			{
				Track track = (Track) o;
				CubicCurve2D cc = transformTrackToView(track);

				if (basePoint == null || !basePoint.getPoint().equals(track.getPoint1()))
				{
					g2.drawImage(imageRedHandler, (int) (cc.getP1().getX() - dWidth),
						(int) (cc.getP1().getY() - dHeight), sce);
				}
				if (basePoint == null || !basePoint.getPoint().equals(track.getPoint2()))
				{
					g2.drawImage(imageRedHandler, (int) (cc.getP2().getX() - dWidth),
						(int) (cc.getP2().getY() - dHeight), sce);
				}
			}
		}

		if (basePoint != null)
		{
			double dHeight = imageBasePoint.getHeight(sce) / 2;
			double dWidth = imageBasePoint.getWidth(sce) / 2;

			Point2D point = sce.modelToView(basePoint.getPoint());
			
			g2.drawImage(imageBasePoint, (int) (point.getX() - dWidth),
				(int) (point.getY() - dHeight), sce);
		}
	}

	/** Flag that indicates that cache is not up to date. */
	private boolean validCachedDisplay;

	/** List of cached tracks. */
	private ArrayList cachedTracks = null;

	/** List of cached switches. */
	private ArrayList cachedSwitches = null;
	
	/** List of cached track names. */
	private ArrayList cachedTrackNames = null;

	/**
	 * Updates cached display, but it does not help much.
	 */
	protected void updateCachedDisplay()
	{
		if (scenery != null)
		{
			scenery.readLock();
			try
			{
				Iterator iterator;
				CachedTrack cachedTrack;
				int n;
				n = scenery.getCount(Track.class);
				cachedTracks.clear();
				cachedTracks.ensureCapacity(n);
				
				iterator = scenery.getIterator(Track.class);
				for (int i = 0; i < n; i++)
				{
					Track track = (Track) iterator.next();
					CubicCurve2D cc = transformTrackToView(track);
					cachedTrack = new CachedTrack(cc);
					cachedTrack.tr = track;
					cachedTracks.add(cachedTrack);
				}

				n = scenery.getCount(Switch.class);
				cachedSwitches.clear();
				cachedTracks.ensureCapacity(n);
				
				iterator = scenery.getIterator(Switch.class);
				for (int i = 0; i < n; i++)
				{
					Switch sswitch = (Switch) iterator.next();
					CubicCurve2D[] cc = transformSwitchToView(sswitch);
					CachedSwitch cachedSwitch = new CachedSwitch(cc[0], cc[1]);
					cachedSwitch.sw = sswitch;
					cachedSwitches.add(cachedSwitch);
				}

				/* Names */
				n = scenery.getTrackNames().getSize();

				cachedTrackNames.clear();
				cachedTrackNames.ensureCapacity(n);
				for (int i = 0; i < n; i++)
				{
					Track t = (Track) scenery.getTrackNames().getElementAt(i);
					CachedTextBox cachedTextBox = new CachedTextBox();
					cachedTextBox.tb = transformNamedTrack(t);
					cachedTextBox.tr = t;
					cachedTrackNames.add(cachedTextBox);
				}

			}
			finally
			{
				scenery.readUnlock();
			}
		}

		validCachedDisplay = true;
	}

	/**
	 * Transforms track to cubic curve.
	 *
	 * @param track Original track from scenery.
	 * @return Cubic curve in component space.
	 */
	protected CubicCurve2D transformTrackToView(Track track)
	{
		Point3d p3d;

		CubicCurve2D cc = new CubicCurve2D.Double();

		Point2D point1 = sce.modelToView(track.getPoint1());
		Point2D point2 = sce.modelToView(track.getPoint2());
		p3d = (Point3d) track.getPoint1().clone();
		p3d.add(track.getVector1());
		Point2D ctrl1 = sce.modelToView(p3d);
		p3d = (Point3d) track.getPoint2().clone();
		p3d.add(track.getVector2());
		Point2D ctrl2 = sce.modelToView(p3d);
		
		cc.setCurve(point1, ctrl1, ctrl2, point2);

		return cc;
	}

	/**
	 * Transforms switch to two cubic curves.
	 *
	 * @param sw Original switch from scenery.
	 * @return Array of two cubic curves.
	 */
	protected CubicCurve2D[] transformSwitchToView(Switch sw)
	{
		Point3d p3d;

		CubicCurve2D[] aCc = new CubicCurve2D[2];
		aCc[0] = new CubicCurve2D.Double();
		aCc[1] = new CubicCurve2D.Double();

		Point2D point1 = sce.modelToView( sw.getPoint1() );
		Point2D point2 = sce.modelToView( sw.getPoint2() );
		p3d = (Point3d) sw.getPoint1().clone();
		p3d.add( sw.getVector1() );
		Point2D ctrl1 = sce.modelToView(p3d);
		p3d = (Point3d) sw.getPoint2().clone();
		p3d.add( sw.getVector2() );
		Point2D ctrl2 = sce.modelToView(p3d);

		aCc[0].setCurve(point1, ctrl1, ctrl2, point2);

		Point2D point3 = sce.modelToView( sw.getPoint1() );
		Point2D point4 = sce.modelToView( sw.getPoint3() );
		p3d = (Point3d) sw.getPoint1().clone();
		p3d.add( sw.getSwitchedVector() );
		Point2D ctrl3 = sce.modelToView(p3d);
		p3d = (Point3d) sw.getPoint3().clone();
		p3d.add( sw.getVector3() );
		Point2D ctrl4 = sce.modelToView(p3d);

		aCc[1].setCurve(point3, ctrl3, ctrl4, point4);

		return aCc;
	}

	/**
	 * Computes text boxes for named tracks.
	 *
	 * @param t Named track.
	 * @return Text box.
	 */
	protected TextBox transformNamedTrack(Track t)
	{
		Font font = new Font("Arial Narrow", 0, 11);
		FontRenderContext frc = 
			((Graphics2D) sce.getGraphics()).getFontRenderContext();

		Point3d startP3D = t.getPoint1();
		Point3d endP3D = t.getPoint2();
		Point3d middleP3d = new Point3d();
		middleP3d.x = (endP3D.x - startP3D.x) / 2;
		middleP3d.y = (endP3D.y - startP3D.y) / 2;
		middleP3d.z = (endP3D.z - startP3D.z) / 2;
		middleP3d.add(startP3D);
		Point2D p2d = sce.modelToView( middleP3d );
		double alfa = Math.atan2(p2d.getY(), p2d.getX());
		double dx = 10 * Math.sin(Math.PI * 0.5 - alfa);
		double dy = 10 * Math.cos(Math.PI * 0.5 - alfa);
		p2d.setLocation(p2d.getX() + dx, p2d.getY() - dy);

		TextBox textBox = new TextBox(t.getName(), font, frc);
		textBox.setLocation( p2d );

		return textBox;
	}

	/**
	 * Determines if the shape should be drawn in clipping
	 * rectangle.
	 */
	protected boolean shouldDraw(Rectangle rect, Shape a)
	{
		if (isOutOfView(rect, a))
			return false;

		if (intersects(rect, a))
			return true;

		return contains(rect, a);
	}

	/**
	 * Checks if the shape is out of rectangle.
	 *
	 * @return <code>True</code>, if the shape should not be drawn.
	 */
	private boolean isOutOfView(Rectangle rect, Shape a)
	{
		double wykl = 10 * sce.getScale() * PIXELS_PER_METER;
		double regMinX = rect.x - wykl;
		double regMaxX = rect.x + rect.width + wykl;
		double regMinY = rect.y - wykl;
		double regMaxY = rect.y + rect.height + wykl;

		if (a instanceof CubicCurve2D)
		{
			CubicCurve2D cc = (CubicCurve2D) a;

			if (cc.getX1() < regMinX && cc.getX2() < regMinX)
				return true;
			if (cc.getX1() > regMaxX && cc.getX2() > regMaxX)
				return true;
			if (cc.getY1() < regMinY && cc.getY2() < regMinY)
				return true;
			if (cc.getY1() > regMaxY && cc.getY2() > regMaxY)
				return true;

			return false;
		}
		else if (a instanceof Rectangle2D)
		{
			Rectangle2D rr = (Rectangle2D) a;

			if (rr.getMinX() < regMinX && rr.getMaxX() < regMinX)
				return true;
			if (rr.getMinX() > regMaxX && rr.getMaxX() > regMaxX)
				return true;
			if (rr.getMinY() < regMinY && rr.getMaxY() < regMinY)
				return true;
			if (rr.getMinY() > regMaxY && rr.getMaxY() > regMaxY)
				return true;

			return false;
		}

		return true;
	}

	/**
	 * Determines that the shape intersects clipping region.
	 * <p>This method is faster than stanard cubic curve intersect
	 * determination, but it may return <code>true</code> though
	 * the shape does not intersect the clipping region.</p>
	 *
	 * @param rect Clipping rectangle.
	 * @param a Shape to display.
	 * @return <code>False</code>, if a shape does not intersects 
	 *   with rectangle.
	 */
	private boolean intersects(Rectangle rect, Shape a)
	{
		if (a instanceof CubicCurve2D)
		{
			CubicCurve2D cc = (CubicCurve2D) a;
			
			double p1x = cc.getX1();
			double p1y = cc.getY1();
			double p2x = cc.getX2();
			double p2y = cc.getY2();
			double c1x = cc.getCtrlX1();
			double c1y = cc.getCtrlY1();
			double c2x = cc.getCtrlX2();
			double c2y = cc.getCtrlY2();
			
			return rect.intersectsLine(p1x, p1y, c1x, c1y) ||
				rect.intersectsLine(c1x, c1y, c2x, c2y) ||
				rect.intersectsLine(c2x, c2y, p2x, p2y) ||
				rect.intersectsLine(p2x, p2y, p1x, p1y);
		}
		else if (a instanceof Rectangle2D)
		{
			Rectangle2D rr = (Rectangle2D) a;

			return rect.intersects(rr);
		}

		return false;
	}

	private boolean contains(Rectangle rect, Shape a)
	{
		if (a instanceof CubicCurve2D)
		{
			CubicCurve2D cc = (CubicCurve2D) a;

			GeneralPath gp = new GeneralPath();
			gp.moveTo((float) cc.getX1(), (float) cc.getY1());
			gp.lineTo((float) cc.getCtrlX1(), (float) cc.getCtrlY1());
			gp.lineTo((float) cc.getCtrlX2(), (float) cc.getCtrlY2());
			gp.lineTo((float) cc.getX2(), (float) cc.getY2());
			gp.lineTo((float) cc.getX1(), (float) cc.getY1());

			//boolean b = cc.contains(rect.x, rect.y) &&
			//	cc.contains(rect.x + rect.width, rect.y) &&
			//	cc.contains(rect.x + rect.width, rect.y + rect.height) &&
			//	cc.contains(rect.x, rect.y + rect.height);

			//return b;

			return gp.contains(rect);
		}
		else if (a instanceof Rectangle2D)
		{
			Rectangle2D rr = (Rectangle2D) a;

			return rr.contains(rect);
		}

		return false;
	}

	/**
	 * Cached track references track on screen (cubic curve) and track in scenery model.
	 * This is because of performance.
	 */
	class CachedTrack
	{
		public CachedTrack()
		{
			cc = new CubicCurve2D.Double();
		}

		public CachedTrack(CubicCurve2D cc)
		{
			this.cc = cc;
		}

		/** Reference to on-screen representation. */
		public CubicCurve2D cc;
		/** Reference to scenery element. */
		public Track tr;
	}

	/**
	 * Cached switch references switch on screen (two cubic curves) and the switch
	 * in scenery model. This is because of performance.
	 */
	class CachedSwitch
	{
		public CachedSwitch()
		{
			cc1 = new CubicCurve2D.Double();
			cc2 = new CubicCurve2D.Double();
		}

		public CachedSwitch(CubicCurve2D cc1, CubicCurve2D cc2)
		{
			this.cc1 = cc1;
			this.cc2 = cc2;
		}

		/** Reference to on-screen representation's first track of the switch. */
		public CubicCurve2D cc1;
		/** Reference to on-screen representation's second track of the switch. */
		public CubicCurve2D cc2;
		/** Reference to the switch in scenery. */
		public Switch sw;
	}

	/**
	 * Cached text box containts text box and named track.
	 */
	class CachedTextBox
	{
		/** Reference to named track. */
		public Track tr;
		/** Reference to text box. */
		public TextBox tb;
	}

	/*
	 * --------------- Dimensions ------------------
	 */

	/**
	 * Gets a preferred size of component.
	 *
	 * @param c Component.
	 * @return Preferred size.
	 */
	public Dimension getPreferredSize(JComponent c)
	{
		Dimension pSize = getMinimumSize(c);

		if (!validPreferredSize)
			updatePreferredSize();
		
		return new Dimension(
			Math.max(pSize.width, preferredSize.width),
			Math.max(pSize.height, preferredSize.height));
	}

	/**
	 * Gets the minimum size of the scenery editor.
	 * <p>Default is <i>(1200, 1200)</i>.</p>
	 *
	 * @return Minimum size.
	 */
	public Dimension getMinimumSize(JComponent c)
	{
		return new Dimension(1200, 1200);
	}

	// ----------- Variables linked with size ----------------

	private double minX = 0.0;
	private double maxX = 0.0;
	private double minY = 0.0;
	private double maxY = 0.0;

	/**
	 * Updates size for the component when data or scale changes.	 
	 */
	protected void updatePreferredSize()
	{
		if (!validMinMaxInfo)
		{
			updateMinMaxInfo();
		}

		preferredSize.width = (int) modelToView(maxX) - (int) modelToView(minX);
		preferredSize.height = (int) modelToView(maxY) - (int) modelToView(minY);
		preferredSize.width += 2 * PIXELS_PER_METER;
		preferredSize.height += 2 * PIXELS_PER_METER;

		validPreferredSize = true;
	}

	/**
	 * Updates when necessary min-max information.
	 */
	protected void updateMinMaxInfo()
	{
		scenery.readLock();
		try
		{
			Iterator iterator;
			minX = maxX = minY = maxY = 0.0;

			iterator = scenery.getIterator(Track.class);
			while (iterator.hasNext())			
			{
				Track t = (Track) iterator.next();

				Point3d p1 = t.getPoint1();
				Point3d p2 = t.getPoint2();
				Point3d c1 = (Point3d) t.getPoint1().clone();
				Point3d c2 = (Point3d) t.getPoint2().clone();
				c1.add(t.getVector1());
				c2.add(t.getVector2());
				minX = Math.min(p1.x, minX);
				minX = Math.min(p2.x, minX);
				minX = Math.min(c1.x, minX);
				minX = Math.min(c2.x, minX);
				maxX = Math.max(p1.x, maxX);
				maxX = Math.max(p2.x, maxX);
				maxX = Math.max(c1.x, maxX);
				maxX = Math.max(c2.x, maxX);
				minY = Math.min(p1.z, minY);
				minY = Math.min(p2.z, minY);
				minY = Math.min(c1.z, minY);
				minY = Math.min(c2.z, minY);
				maxY = Math.max(p1.z, maxY);
				maxY = Math.max(p2.z, maxY);
				maxY = Math.max(c1.z, maxY);
				maxY = Math.max(c2.z, maxY);				
			}

			iterator = scenery.getIterator(Switch.class);
			while (iterator.hasNext())
			{
				Switch sswitch = (Switch) iterator.next();

				Point3d p1 = sswitch.getPoint1();
				Point3d p2 = sswitch.getPoint2();
				Point3d p3 = sswitch.getPoint3();
				minX = Math.min(p1.x, minX);
				minX = Math.min(p2.x, minX);
				minX = Math.min(p3.x, minX);
				maxX = Math.max(p1.x, maxX);
				maxX = Math.max(p2.x, maxX);
				maxX = Math.max(p3.x, maxX);
				minY = Math.min(p1.z, minY);
				minY = Math.min(p2.z, minY);
				minY = Math.min(p3.z, minY);
				maxY = Math.max(p1.z, maxY);
				maxY = Math.max(p2.z, maxY);
				maxY = Math.max(p3.z, maxY);				
			}

			validMinMaxInfo = true;
		}
		finally
		{
			scenery.readUnlock();
		}
	}

	/*
	 * -------------- Editor kits ----------------
	 */
	
	/**
	 * Gets an editor kit for component.
	 *
	 * @param sc Scenery editor component.
	 * @return Editor kit.
	 * @see EditorKit
	 */
	public EditorKit getEditorKit(JSceneryEditor sc)
	{
		return defaultKit;
	}

	/*
	 * ---------------- Model / view --------------
	 */
	
	/**
	 * Translates model coordinates to view coordinates for specfied
	 * scenery editor.
	 *
	 * @param p Point in model coordinates.
	 * @return Point in component coordinates.
	 */
	public Point2D modelToView(JSceneryEditor editor, Point3d p)
	{		
		int dist;
		Point2D.Double point = new Point2D.Double();

		Dimension dim = getMinimumSize(editor);

		point.x = maxX - p.x;
		point.x *= sce.getScale() * PIXELS_PER_METER;
		dist = (int) modelToView(maxX - minX);
		if (dist <= dim.width - 2 * PIXELS_PER_METER)
		{
			point.x += (dim.width - dist) / 2;
		}
		else
		{
			point.x += PIXELS_PER_METER;
		}

		point.y = maxY - p.z;
		point.y *= sce.getScale() * PIXELS_PER_METER;
		dist = (int) modelToView(maxY - minY);
		if (dist <= dim.height - 2 * PIXELS_PER_METER)
		{
			point.y += (dim.height - dist) / 2;
		}
		else
		{
			point.y += PIXELS_PER_METER;
		}

		return point;
	}

	/**
	 * Computes single double from model value to view value.
	 */
	private double modelToView(double d)
	{
		return d * sce.getScale() * PIXELS_PER_METER;
	}

	/**
	 * Translates view coordinates (coordinates within GUI component)
	 * to model coordinates in three dimensonal space.
	 *
	 * @param editor Editor.
	 * @param p Point in view coordinates.
	 * @return Point in model corrdinates.
	 */
	public Point3d viewToModel(JSceneryEditor editor, Point2D p)
	{
		double px = p.getX();
		double py = p.getY();
		int dist;
		Dimension mSize = getMinimumSize(editor);
		Point3d point = new Point3d();

		dist = (int) modelToView(maxX - minX);
		if (mSize.width - 2 * PIXELS_PER_METER >= dist)
		{
			point.x = px - (mSize.width - dist) / 2;
		}
		else
		{
			point.x = px - PIXELS_PER_METER;
		}
		point.x /= (sce.getScale() * PIXELS_PER_METER);
		point.x = maxX - point.x;

		dist = (int) modelToView(maxY - minY);
		if (mSize.height - 2 * PIXELS_PER_METER >= dist)
		{
			point.z = py - (mSize.height - dist) / 2;
		}
		else
		{
			point.z = py - PIXELS_PER_METER;
		}
		point.z /= (sce.getScale() * PIXELS_PER_METER);
		point.z = maxY - point.z;

		return point;
	}

	/**
	 * Returns rectangle that contains minimum X, maximum X, minimum Y,
	 * maximum Y values in model coordinates' layout, but the axes have 
	 * view meaning.
	 * <p>Returned rectangle bounds scenery.</p>
	 * <p><i>X</i> variable of rectangle means maximum X, <i>y</i>
	 * variable means maximum Y. <i>Width</i> (negative value) of rectangle is 
	 * a distance, between maxmium X and minimum X, so <i>x</i> + <i>width</i>
	 * equals minimum Y. <i>Height</i> (negative value) variable in rectangle 
	 * is a distance between the maximum Y and the minimum Y, so <i>y</i> + 
	 * <i>height</i> equals the minimum Y. You can easly obtain minimums and 
	 * maximums using <code>getMinX()<code>, <code>getMaxX()</code>, 
	 * <code>getMinY()</code> and <code>getMaxY()</code> rectangle's methods.</p>
	 *
	 * @return Rectangle bound scenery.
	 */
	public Rectangle2D queryMinMax()
	{
		Rectangle2D.Double rect = new Rectangle2D.Double();
		rect.x = maxX;
		rect.y = maxY;
		rect.width = minX - maxX;
		rect.height = minY - maxY;
		return rect;
	}

	/*
	 * ---------------- Colors ----------------------
	 */
	
	/**
	 * Sets the track color.
	 *
	 * @param c New color of the track.
	 */
	public void setTrackColor(Color c)
	{
		trackColor = c;
	}

	/**
	 * Gets the track color.
	 *
	 * @return Current track color.
	 */
	public Color getTrackColor()
	{
		return trackColor;
	}

	/**
	 * Sets the switch color.
	 *
	 * @param c New color of the switch.
	 */
	public void setSwitchColor(Color c)
	{
		switchColor = c;
	}

	/**
	 * Gets the switch color.
	 *
	 * @return Current color of the switch.
	 */
	public Color getSwitchColor()
	{
		return switchColor;
	}

	/**
	 * Sets track names color.
	 *
	 * @param c The new color.
	 */
	public void setTrackNamesColor(Color c)
	{
		trackNamesColor = c;
	}

	/**
	 * Gets track names color.
	 *
	 * @return Current color of the track names.
	 */
	public Color getTrackNamesColor()
	{
		return trackNamesColor;
	}

	/**
	 * Sets the visibility of track names.
	 *
	 * @param b If <code>true</code>, track names will be visible.
	 */
	public void setTrackNamesVisibility(boolean b)
	{
		if (trackNamesVisibility != b)
		{
			trackNamesVisibility = b;
			sce.repaint();
		}
	}

	/**
	 * Gets track names visibility flag.
	 *
	 * @return <code>True</code>, if track names are visible.
	 */
	public boolean getTrackNamesVisibility()
	{
		return trackNamesVisibility;
	}

	/*
	 * ----------- Listeners classes ----------------
	 */

	/**
	 * Property change listener listens component property changes.
	 */
	public class PropertyChangeHandler
		implements PropertyChangeListener
	{
		public void propertyChange(PropertyChangeEvent event)
		{
			if (event.getSource() == sce)
			{
				String name = event.getPropertyName();
				if (name.equals(JSceneryEditor.SCENERY_MODEL_PROPERTY))
				{
					setModel((Scenery) event.getNewValue());
				}
				else if (name.equals(JSceneryEditor.EDITABLE_PROPERTY))
				{
					boolean editable = ((Boolean) event.getNewValue()).booleanValue();
					if (editable)
					{
						Cursor cursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
						sce.setCursor(cursor);
						oldCursor = cursor;
					}
					else
					{
						Cursor cursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
						sce.setCursor(cursor);
						oldCursor = cursor;
					}
				}
				else if (name.equals(JSceneryEditor.GRID_VISIBLE_PROPERTY))
				{
					sce.repaint( sce.getVisibleRect() );
				}
				else if (name.equals(JSceneryEditor.SCALE_PROPERTY))
				{
					validPreferredSize = false;
					validCachedDisplay = false;
					sce.revalidate();
					sce.repaint( sce.getVisibleRect() );
				}
				else if (name.equals(JSceneryEditor.SCENERY_SELECTION_PROPERTY))
				{
					setSelectionModel(sce.getSelectionModel());
				}
				else if (name.equals(JSceneryEditor.BASEPOINT_PROPERTY))
				{
					processBasePoint((BasePoint) event.getOldValue(),
						(BasePoint) event.getNewValue());
				}
			}
		}

		private void processBasePoint(BasePoint o, BasePoint n)
		{
			// Przerysuj stary
			if (o != null)
			{
				Rectangle rect = new Rectangle();
				Point2D p = sce.modelToView( o.getPoint() );
				rect.x = (int) p.getX();
				rect.y = (int) p.getY();
				rect.grow(6, 6);
				sce.repaint(rect);
			}

			if (n != null)
			{
			// sprawd czy nie wyszlimy za zakres
			double oldMinX = minX;
			double oldMaxX = maxX;
			double oldMinY = minY;
			double oldMaxY = maxY;

			Point3d p3d = n.getPoint();
			minX = Math.min(minX, p3d.x);
			maxX = Math.max(maxX, p3d.x);
			minY = Math.min(minY, p3d.z);
			maxY = Math.max(maxY, p3d.z);

			// Sprawd czy zaszy zmiany
			if (oldMinX != minX || oldMaxX != maxX || oldMinY != minY || oldMaxY != maxY)
			{
				int nTracks = cachedTracks.size();
				for (int i = 0; i < nTracks; i++)
				{
					CachedTrack cachedTrack = (CachedTrack) cachedTracks.get(i);
					CubicCurve2D cc = transformTrackToView(cachedTrack.tr);
					cachedTrack.cc = cc;
				}
				int nSwitches = cachedSwitches.size();
				for (int i = 0; i < nSwitches; i++)
				{
					CachedSwitch cachedSwitch = (CachedSwitch) cachedSwitches.get(i);
					CubicCurve2D[] cc = transformSwitchToView(cachedSwitch.sw);
					cachedSwitch.cc1 = cc[0];
					cachedSwitch.cc2 = cc[1];
				}

				// No i zostay napisy do zrobienia
				int nTextBoxes = cachedTrackNames.size();
				for (int i = 0; i < nTextBoxes; i++)
				{
					CachedTextBox cachedTextBox = (CachedTextBox) cachedTrackNames.get(i);
					cachedTextBox.tb = transformNamedTrack(cachedTextBox.tr);
				}

				validMinMaxInfo = true;
				validPreferredSize = false;
				sce.revalidate();
			}

			sce.repaint( sce.getVisibleRect() );
			}
		}
	}

	/**
	 * Scenery listener.
	 */
	public class SceneryChangeHandler implements SceneryChangeListener
	{
		public void sceneryChanged(SceneryEvent sEvent)
		{
			int type = sEvent.getType();
			if (type == SceneryEvent.CHANGE)
			{
				change(sEvent);
			}
			else if (type == SceneryEvent.REMOVE)
			{
				remove(sEvent);
			}
			else if (type == SceneryEvent.INSERT)
			{
				insert(sEvent);
			}
		}

		private void remove(SceneryEvent sEvent)
		{
			Rectangle oldRect = new Rectangle();
			int n;

			Object[] remObjects = sEvent.getRemoved();
			n = remObjects.length;
			for (int i = 0; i < n; i++)
			{
				if (remObjects[i] instanceof Switch)
				{
					Switch sswitch = (Switch) remObjects[i];

					// Znajd w cache rozjazd i usu go.
					int nSwitches = cachedSwitches.size();
					for (int j = 0; j < nSwitches; j++)
					{
						CachedSwitch cachedSwitch = (CachedSwitch) cachedSwitches.get(j);
						if (cachedSwitch.sw == sswitch)
						{
							oldRect.add(cachedSwitch.cc1.getBounds());
							oldRect.add(cachedSwitch.cc2.getBounds());
							cachedSwitches.remove(j);
							break;
						}
					}
				}
				else if (remObjects[i] instanceof Track)
				{
					Track track = (Track) remObjects[i];

					// Znajd w cache tor i usu go.
					int nTracks = cachedTracks.size();
					for (int j = 0; j < nTracks; j++)
					{
						CachedTrack cachedTrack = (CachedTrack) cachedTracks.get(j);
						if (cachedTrack.tr == track)
						{
							oldRect.add(cachedTrack.cc.getBounds());
							cachedTracks.remove(j);
							break;
						}
					}
				}

				// Usu nazwy torw
				if (remObjects[i] instanceof Track &&
					((Track) remObjects[i]).getName() != null)
				{
					Track track = (Track) remObjects[i];

					int nNames = cachedTrackNames.size();
					for (int j = 0; j < nNames; j++)
					{
						CachedTextBox textBox = (CachedTextBox) cachedTrackNames.get(j);
						if (textBox.tr == track)
						{
							oldRect.add(textBox.tb.getBounds());
							cachedTrackNames.remove(j);
							break;
						}
					}
				}
			}

			Rectangle visibleRect = sce.getVisibleRect();
			sce.repaint( visibleRect.intersection(oldRect) );
		}

		private void change(SceneryEvent sEvent)
		{
			Rectangle oldRect = new Rectangle();
			Rectangle newRect = new Rectangle();
			int n;

			Object[] oldObjects = sEvent.getRemoved();
			Object[] newObjects = sEvent.getInserted();

			// Znajd usunite cachowane obiekty i usu je.
			n = oldObjects.length;
			for (int i = 0; i < n; i++)
			{
				if (oldObjects[i] instanceof Switch)
				{
					Switch sswitch = (Switch) oldObjects[i];

					// Znajd w cache rozjazd i usu go.
					int nSwitches = cachedSwitches.size();
					for (int j = 0; j < nSwitches; j++)
					{
						CachedSwitch cachedSwitch = (CachedSwitch) cachedSwitches.get(j);
						if (cachedSwitch.sw == sswitch)
						{
							oldRect.add(cachedSwitch.cc1.getBounds());
							oldRect.add(cachedSwitch.cc2.getBounds());
							cachedSwitches.remove(j);
							break;
						}
					}
				}
				else if (oldObjects[i] instanceof Track)
				{
					Track track = (Track) oldObjects[i];

					// Znajd w cache tor i usu go.
					int nTracks = cachedTracks.size();
					for (int j = 0; j < nTracks; j++)
					{
						CachedTrack cachedTrack = (CachedTrack) cachedTracks.get(j);
						if (cachedTrack.tr == track)
						{
							oldRect.add(cachedTrack.cc.getBounds());
							cachedTracks.remove(j);
							break;
						}
					}
				}

				// Usu nazwy torw
				if (oldObjects[i] instanceof Track &&
					((Track) oldObjects[i]).getName() != null)
				{
					Track track = (Track) oldObjects[i];

					int nNames = cachedTrackNames.size();
					for (int j = 0; j < nNames; j++)
					{
						CachedTextBox textBox = (CachedTextBox) cachedTrackNames.get(j);
						if (textBox.tr == track)
						{
							oldRect.add(textBox.tb.getBounds());
							cachedTrackNames.remove(j);
							break;
						}
					}
				}
			}

			// Sprawd czy nie trzeba zwikszy maxw i minw.
			// Zmniejszanie olaem
			double oldMinX = minX;
			double oldMaxX = maxX;
			double oldMinY = minY;
			double oldMaxY = maxY;

			n = newObjects.length;
			for (int i = 0; i < n; i++)
			{
				if (newObjects[i] instanceof Switch)
				{
				Switch sswitch = (Switch) newObjects[i];

				Point3d p1 = sswitch.getPoint1();
				Point3d p2 = sswitch.getPoint2();
				Point3d p3 = sswitch.getPoint3();
				minX = Math.min(p1.x, minX);
				minX = Math.min(p2.x, minX);
				minX = Math.min(p3.x, minX);
				maxX = Math.max(p1.x, maxX);
				maxX = Math.max(p2.x, maxX);
				maxX = Math.max(p3.x, maxX);
				minY = Math.min(p1.z, minY);
				minY = Math.min(p2.z, minY);
				minY = Math.min(p3.z, minY);
				maxY = Math.max(p1.z, maxY);
				maxY = Math.max(p2.z, maxY);
				maxY = Math.max(p3.z, maxY);				
				}
				else if (newObjects[i] instanceof Track)
				{
				Track t = (Track) newObjects[i];

				Point3d p1 = t.getPoint1();
				Point3d p2 = t.getPoint2();
				Point3d c1 = (Point3d) t.getPoint1().clone();
				Point3d c2 = (Point3d) t.getPoint2().clone();
				c1.add(t.getVector1());
				c2.add(t.getVector2());
				minX = Math.min(p1.x, minX);
				minX = Math.min(p2.x, minX);
				minX = Math.min(c1.x, minX);
				minX = Math.min(c2.x, minX);
				maxX = Math.max(p1.x, maxX);
				maxX = Math.max(p2.x, maxX);
				maxX = Math.max(c1.x, maxX);
				maxX = Math.max(c2.x, maxX);
				minY = Math.min(p1.z, minY);
				minY = Math.min(p2.z, minY);
				minY = Math.min(c1.z, minY);
				minY = Math.min(c2.z, minY);
				maxY = Math.max(p1.z, maxY);
				maxY = Math.max(p2.z, maxY);
				maxY = Math.max(c1.z, maxY);
				maxY = Math.max(c2.z, maxY);
				}
			}

			// Sprawd czy zaszy zmiany
			if (oldMinX != minX || oldMaxX != maxX || oldMinY != minY || oldMaxY != maxY)
			{
				int nTracks = cachedTracks.size();
				for (int i = 0; i < nTracks; i++)
				{
					CachedTrack cachedTrack = (CachedTrack) cachedTracks.get(i);
					CubicCurve2D cc = transformTrackToView(cachedTrack.tr);
					cachedTrack.cc = cc;
				}
				int nSwitches = cachedSwitches.size();
				for (int i = 0; i < nSwitches; i++)
				{
					CachedSwitch cachedSwitch = (CachedSwitch) cachedSwitches.get(i);
					CubicCurve2D[] cc = transformSwitchToView(cachedSwitch.sw);
					cachedSwitch.cc1 = cc[0];
					cachedSwitch.cc2 = cc[1];
				}

				// No i zostay napisy do zrobienia
				int nTextBoxes = cachedTrackNames.size();
				for (int i = 0; i < nTextBoxes; i++)
				{
					CachedTextBox cachedTextBox = (CachedTextBox) cachedTrackNames.get(i);
					cachedTextBox.tb = transformNamedTrack(cachedTextBox.tr);
				}

				validMinMaxInfo = true;
				validPreferredSize = false;
				sce.revalidate();
			}


			// Dodaj nowe obiekty do cache-w.
			for (int i = 0; i < n; i++)
			{
				if (newObjects[i] instanceof Switch)
				{
					Switch sswitch = (Switch) newObjects[i];
					CubicCurve2D[] cc = transformSwitchToView(sswitch);
					CachedSwitch cachedSwitch = new CachedSwitch(cc[0], cc[1]);
					cachedSwitch.sw = sswitch;
					cachedSwitches.add(cachedSwitch);
					newRect.add(cachedSwitch.cc1.getBounds());
					newRect.add(cachedSwitch.cc2.getBounds());
				}
				else if (newObjects[i] instanceof Track)
				{
					Track track = (Track) newObjects[i];
					CubicCurve2D cc = transformTrackToView(track);
					CachedTrack cachedTrack = new CachedTrack(cc);
					cachedTrack.tr = track;
					cachedTracks.add(cachedTrack);
					newRect.add(cachedTrack.cc.getBounds());
				}

				// Dodaj nazwy torw
				if (newObjects[i] instanceof Track &&
					((Track) newObjects[i]).getName() != null)
				{
					Track track = (Track) newObjects[i];
					CachedTextBox cachedTextBox = new CachedTextBox();
					cachedTextBox.tb = transformNamedTrack(track);
					cachedTextBox.tr = track;
					cachedTrackNames.add(cachedTextBox);
				}
			}

			Rectangle sumRect = oldRect.union(newRect);
			Rectangle visibleRect = sce.getVisibleRect();

			/*
			 * Przerysuj cay ekran (efektywno zaley od warunkw).
			 * Dla maych powiksze to jest na pewno nieefektywne.
			 */
			//sce.repaint( visibleRect.intersection(sumRect) );
			sce.repaint( visibleRect );
		}

		private void insert(SceneryEvent sEvent)
		{
			Rectangle newRect = new Rectangle();
			int n;

			Object[] newObjects = sEvent.getInserted();

			// Zmniejszanie olaem
			double oldMinX = minX;
			double oldMaxX = maxX;
			double oldMinY = minY;
			double oldMaxY = maxY;

			n = newObjects.length;
			for (int i = 0; i < n; i++)
			{
				if (newObjects[i] instanceof Switch)
				{
				Switch sswitch = (Switch) newObjects[i];

				Point3d p1 = sswitch.getPoint1();
				Point3d p2 = sswitch.getPoint2();
				Point3d p3 = sswitch.getPoint3();
				minX = Math.min(p1.x, minX);
				minX = Math.min(p2.x, minX);
				minX = Math.min(p3.x, minX);
				maxX = Math.max(p1.x, maxX);
				maxX = Math.max(p2.x, maxX);
				maxX = Math.max(p3.x, maxX);
				minY = Math.min(p1.z, minY);
				minY = Math.min(p2.z, minY);
				minY = Math.min(p3.z, minY);
				maxY = Math.max(p1.z, maxY);
				maxY = Math.max(p2.z, maxY);
				maxY = Math.max(p3.z, maxY);				
				}
				else if (newObjects[i] instanceof Track)
				{
				Track t = (Track) newObjects[i];

				Point3d p1 = t.getPoint1();
				Point3d p2 = t.getPoint2();
				Point3d c1 = (Point3d) t.getPoint1().clone();
				Point3d c2 = (Point3d) t.getPoint2().clone();
				c1.add(t.getVector1());
				c2.add(t.getVector2());
				minX = Math.min(p1.x, minX);
				minX = Math.min(p2.x, minX);
				minX = Math.min(c1.x, minX);
				minX = Math.min(c2.x, minX);
				maxX = Math.max(p1.x, maxX);
				maxX = Math.max(p2.x, maxX);
				maxX = Math.max(c1.x, maxX);
				maxX = Math.max(c2.x, maxX);
				minY = Math.min(p1.z, minY);
				minY = Math.min(p2.z, minY);
				minY = Math.min(c1.z, minY);
				minY = Math.min(c2.z, minY);
				maxY = Math.max(p1.z, maxY);
				maxY = Math.max(p2.z, maxY);
				maxY = Math.max(c1.z, maxY);
				maxY = Math.max(c2.z, maxY);
				}
			}

			// Sprawd czy zaszy zmiany
			if (oldMinX != minX || oldMaxX != maxX || oldMinY != minY || oldMaxY != maxY)
			{
				int nTracks = cachedTracks.size();
				for (int i = 0; i < nTracks; i++)
				{
					CachedTrack cachedTrack = (CachedTrack) cachedTracks.get(i);
					CubicCurve2D cc = transformTrackToView(cachedTrack.tr);
					cachedTrack.cc = cc;
				}
				int nSwitches = cachedSwitches.size();
				for (int i = 0; i < nSwitches; i++)
				{
					CachedSwitch cachedSwitch = (CachedSwitch) cachedSwitches.get(i);
					CubicCurve2D[] cc = transformSwitchToView(cachedSwitch.sw);
					cachedSwitch.cc1 = cc[0];
					cachedSwitch.cc2 = cc[1];
				}

				// No i zostay napisy do zrobienia
				int nTextBoxes = cachedTrackNames.size();
				for (int i = 0; i < nTextBoxes; i++)
				{
					CachedTextBox cachedTextBox = (CachedTextBox) cachedTrackNames.get(i);
					cachedTextBox.tb = transformNamedTrack(cachedTextBox.tr);
				}

				validMinMaxInfo = true;
				validPreferredSize = false;
				sce.revalidate();
			}


			// Dodaj nowe obiekty do cache-w.
			for (int i = 0; i < n; i++)
			{
				if (newObjects[i] instanceof Switch)
				{
					Switch sswitch = (Switch) newObjects[i];
					CubicCurve2D[] cc = transformSwitchToView(sswitch);
					CachedSwitch cachedSwitch = new CachedSwitch(cc[0], cc[1]);
					cachedSwitch.sw = sswitch;
					cachedSwitches.add(cachedSwitch);
					newRect.add(cachedSwitch.cc1.getBounds());
					newRect.add(cachedSwitch.cc2.getBounds());
				}
				else if (newObjects[i] instanceof Track)
				{
					Track track = (Track) newObjects[i];
					CubicCurve2D cc = transformTrackToView(track);
					CachedTrack cachedTrack = new CachedTrack(cc);
					cachedTrack.tr = track;
					cachedTracks.add(cachedTrack);
					newRect.add(cachedTrack.cc.getBounds());
				}

				// Dodaj nazwy torw
				if (newObjects[i] instanceof Track &&
					((Track) newObjects[i]).getName() != null)
				{
					Track track = (Track) newObjects[i];
					CachedTextBox cachedTextBox = new CachedTextBox();
					cachedTextBox.tb = transformNamedTrack(track);
					cachedTextBox.tr = track;
					cachedTrackNames.add(cachedTextBox);
				}
			}

			Rectangle visibleRect = sce.getVisibleRect();

			/*
			 * Przerysuj cay ekran (efektywno zaley od warunkw).
			 * Dla maych powiksze to jest na pewno nieefektywne.
			 */
			//sce.repaint( visibleRect.intersection(sumRect) );
			sce.repaint( visibleRect );

		}
	}

	/**
	 * Handles mouse events.
	 */
	public class HighlighterHandler extends MouseAdapter
	{
		public void mouseClicked(MouseEvent me)
		{
			int mode = getMouseMode();

			if (mode == ZOOM_MODE)
			{
				processZooming(me);
			}
			else if (mode == NORMAL_MODE)
			{
				// jeszcze trzeba sprawdzi czy uchwyt nie czy ju torw.
				boolean success;
				Rectangle rect;
				Point p = me.getPoint();
				BitSet result = new BitSet(3);

				if (!selectionModel.isSelectionEmpty())
				{
					// Sprawd czy punkt nie trafi w uchwyt
					Object obj = selectionModel.getSelectedObject();

					if (obj instanceof Switch)
					{
						Point2D p2;
						Switch sswitch = (Switch) obj;

						// Point 1
						p2 = sce.modelToView( sswitch.getPoint1() );
						rect = new Rectangle((int) p2.getX(), (int) p2.getY(), 0, 0);
						rect.grow(4, 4);
						success = rect.contains(p);
						result.set(0, success);
						if (success)
							processBasePointSwitch(sswitch, sswitch.getPoint1());

						// Point 2
						p2 = sce.modelToView( sswitch.getPoint2() );
						rect = new Rectangle((int) p2.getX(), (int) p2.getY(), 0, 0);
						rect.grow(4, 4);
						success = rect.contains(p);
						result.set(1, success);
						if (success)
							processBasePointSwitch(sswitch, sswitch.getPoint2());
						
						// Point 3
						p2 = sce.modelToView( sswitch.getPoint3() );
						rect = new Rectangle((int) p2.getX(), (int) p2.getY(), 0, 0);
						rect.grow(4, 4);
						success = rect.contains(p);
						result.set(2, success);
						if (success)
							processBasePointSwitch(sswitch, sswitch.getPoint3());
					}
					else if (obj instanceof Track)
					{
						Point2D p2;
						Track track = (Track) obj;

						// Point 1
						p2 = sce.modelToView( track.getPoint1() );
						rect = new Rectangle((int) p2.getX(), (int) p2.getY(), 0, 0);
						rect.grow(4, 4);
						success = rect.contains(p);
						result.set(0, success);
						if (success)
							processBasePointTrack(track, track.getPoint1());

						// Point 2
						p2 = sce.modelToView( track.getPoint2() );
						rect = new Rectangle((int) p2.getX(), (int) p2.getY(), 0, 0);
						rect.grow(4, 4);
						success = rect.contains(p);
						result.set(1, success);
						if (success)
							processBasePointTrack(track, track.getPoint2());

						result.set(2, false);
					}					
				}

				if (! (result.get(0) || result.get(1) || result.get(2)) )
					processSelection(me);
			}

		}

		private void processBasePointTrack(Track track, Point3d point)
		{
			Point3d p1 = track.getPoint1();
			Point3d p2 = track.getPoint2();

			if (p1 != point && p2 != point)
				throw new IllegalArgumentException();

			// Sprawdzi czy punkt jest wolny
			Iterator itrt = cachedTracks.iterator();
			while (itrt.hasNext())
			{
				CachedTrack tr = (CachedTrack) itrt.next();
				if (tr.tr != track)
				{
					if (tr.tr.getPoint1().epsilonEquals(point, 0.001) ||
						tr.tr.getPoint2().epsilonEquals(point, 0.001))
						return;
				}
			}
			itrt = cachedSwitches.iterator();
			while (itrt.hasNext())
			{
				CachedSwitch sw = (CachedSwitch) itrt.next();
				if (sw.sw.getPoint1().epsilonEquals(point, 0.001) ||
					sw.sw.getPoint2().epsilonEquals(point, 0.001) ||
					sw.sw.getPoint3().epsilonEquals(point, 0.001))
					return;
			}

			Vector3d resolved = null;

			if (!track.isStraight())
			{
				if (point == p1)
				{
					resolved = new Vector3d( track.getVector1() );
					resolved.negate();
				}
				if (point == p2)
				{
					resolved = new Vector3d( track.getVector2() );
					resolved.negate();
				}
			}
			else
			{
				resolved = new Vector3d();
				if (point == p2)
				{
					resolved.x = p2.x - p1.x;
					resolved.y = p2.y - p1.y;
					resolved.z = p2.z - p1.z;
				}
				else
				{
					resolved.x = p1.x - p2.x;
					resolved.y = p1.y - p2.y;
					resolved.z = p1.z - p2.z;
				}
			}

			BasePoint basePoint = new BasePoint(point, resolved);
			sce.setBasePoint(basePoint);
		}

		private void processBasePointSwitch(Switch sswitch, Point3d point)
		{
			Point3d p1 = sswitch.getPoint1();
			Point3d p2 = sswitch.getPoint2();
			Point3d p3 = sswitch.getPoint3();

			if (p1 != point && p2 != point && p3 != point)
				throw new IllegalArgumentException();

			// Sprawdzi czy punkt jest wolny
			Iterator itrt = cachedSwitches.iterator();
			while (itrt.hasNext())
			{
				CachedSwitch sw = (CachedSwitch) itrt.next();
				if (sw.sw != sswitch)
				{
					if (sw.sw.getPoint1().epsilonEquals(point, 0.001) ||
						sw.sw.getPoint2().epsilonEquals(point, 0.001) ||
						sw.sw.getPoint3().epsilonEquals(point, 0.001))
						return;
				}
			}
			itrt = cachedTracks.iterator();
			while (itrt.hasNext())
			{
				CachedTrack tr = (CachedTrack) itrt.next();
				if (tr.tr.getPoint1().epsilonEquals(point, 0.001) ||
					tr.tr.getPoint2().epsilonEquals(point, 0.001))
					return;
			}

			Vector3d resolved = null;

			if (!sswitch.isStraight())
			{
				if (point == p1)
				{
					resolved = new Vector3d( sswitch.getVector1() );
					resolved.negate();
				}
				if (point == p2)
				{
					resolved = new Vector3d( sswitch.getVector2() );
					resolved.negate();
				}
			}
			else
			{
				resolved = new Vector3d();
				if (point == p2)
				{
					resolved.x = p2.x - p1.x;
					resolved.y = p2.y - p1.y;
					resolved.z = p2.z - p1.z;
				}
				else
				{
					resolved.x = p1.x - p2.x;
					resolved.y = p1.y - p2.y;
					resolved.z = p1.z - p2.z;
				}
			}

			if (!sswitch.isSideStraight())
			{
				if (point == p3)
				{
					resolved = new Vector3d( sswitch.getVector3() );
					resolved.negate();
				}
			}
			else
			{
				resolved = new Vector3d();
				if (point == p3)
				{
					resolved.x = p3.x - p1.x;
					resolved.y = p3.y - p1.y;
					resolved.z = p3.z - p1.z;
				}
			}

			BasePoint basePoint = new BasePoint(point, resolved);
			sce.setBasePoint(basePoint);
		}
		
		public void mousePressed(MouseEvent me)
		{
			if (!sce.hasFocus())
				sce.requestFocus();
		}

		/**
		 * Process track selection.
		 */
		private void processSelection(MouseEvent me)
		{
			int n;
			Point p = me.getPoint();

			boolean selected = false;
			Iterator iterator;
			Stroke stroke = new BasicStroke(6f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
			Rectangle visibleRect = sce.getVisibleRect();

			iterator = cachedTracks.iterator();
			while (iterator.hasNext() && !selected)
			{
				CachedTrack cachedTrack = (CachedTrack) iterator.next();

				if (!isOutOfView(visibleRect, cachedTrack.cc))
				{
					Shape s = stroke.createStrokedShape(cachedTrack.cc);
					if (s.contains(p))
					{
						Track track = null;
						if (sce.isEditable())
						{
							track = cachedTrack.tr;
						}
						else
						{
							track = new ImmutableTrack(cachedTrack.tr);
						}
						selectionModel.setSelectedObject(cachedTrack.tr);
						selected = true;
					}
				}
			}

			iterator = cachedSwitches.iterator();
			while (iterator.hasNext() && !selected)
			{
				CachedSwitch cachedSwitch = (CachedSwitch) iterator.next();

				if (!isOutOfView(visibleRect, cachedSwitch.cc1) ||
					!isOutOfView(visibleRect, cachedSwitch.cc2))
				{
					Shape s1 = stroke.createStrokedShape(cachedSwitch.cc1);
					Shape s2 = stroke.createStrokedShape(cachedSwitch.cc2);
					if (s1.contains(p) || s2.contains(p))
					{
						Switch _switch = null;
						if (sce.isEditable())
						{
							_switch = cachedSwitch.sw;
						}
						else
						{
							_switch = new ImmutableSwitch(cachedSwitch.sw);
						}
						selectionModel.setSelectedObject(cachedSwitch.sw);
						selected = true;
					}
				}
			}
		
		}

		/**
		 * Process zooming in and out by the mouse.
		 */
		private void processZooming(MouseEvent me)
		{
			Point p = me.getPoint();

			if (sce.isEnabled())
			{
				int button = me.getButton();
				double scale = sce.getScale();

				if (button == MouseEvent.BUTTON1)
				{
					scale = incScale(scale);
					Point3d p3d = sce.viewToModel( p );
					sce.setScale( scale );
					sce.centerViewAt( sce.modelToView(p3d) );
				}
				else if (button == MouseEvent.BUTTON3)
				{
					scale = decScale(scale);
					Point3d p3d = sce.viewToModel( p );
					sce.setScale( scale );
					sce.centerViewAt( sce.modelToView(p3d) );
				}
			}			
		}
	}

	/**
	 * Key listener.
	 */
	public class KeyHandler extends KeyAdapter
	{
		/** Key code that is being generated for. */
		protected Action repeatKeyAction;

		private Cursor oldCursor = null;

		public void keyPressed(KeyEvent ke)
		{
			if (sce != null && sce.hasFocus() && sce.isEnabled())
			{
				KeyStroke keyStroke =
					KeyStroke.getKeyStroke(ke.getKeyCode(), ke.getModifiers());

				if (sce.getConditionForKeyStroke(keyStroke)
					== JComponent.WHEN_FOCUSED)
				{
					ActionListener listener =
						sce.getActionForKeyStroke(keyStroke);

					if (listener instanceof Action)
						repeatKeyAction = (Action) listener;
					else
						repeatKeyAction = null;
				}

				if (repeatKeyAction != null)
				{
					repeatKeyAction.actionPerformed(
						new ActionEvent(
							sce,
							ActionEvent.ACTION_PERFORMED,
							""));
					ke.consume();
				}

				/*
				 * Escape causes selection clearing.
				 */
				if (ke.getKeyCode() == KeyEvent.VK_ESCAPE)
				{
					selectionModel.clearSelection();
				}
			}
		}
	}

	/** Scale constans. */
	private	static double[] scales = {
		1.0, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01, 0.005, 0.002, 0.001,
		5.0E-4, 2.0E-4, 1.0E-4
	};

	/**
	 * Increments the scale.
	 */
	private double incScale(double scale)
	{
		int i = scales.length - 1;

		if (scale == 1.0)
			return scale;

		while (i > 0 && scale >= scales[i])
		{
			i--;
		}

		return scales[i];
	}

	/**
	 * Decrements the scale.
	 */
	private double decScale(double scale)
	{
		int i = 1;
		int n = scales.length;

		if (scale < scales[n-1])
			return scale;

		while (i < n && scale <= scales[i])
		{
			i++;
		}

		if (i == n)
			return scales[i-1];

		return scales[i];
	}

	/**
	 * Scenery selection listener.
	 */
	public class SelectionHandler implements ScenerySelectionListener
	{
		public void valueChanged(ScenerySelectionEvent sse)
		{
			// Repaint old selection
			ScenerySelectionModel ssm = sse.getSelectionModel();
			if (ssm instanceof MemorableSelectionModel)
			{
				Object obj = ((MemorableSelectionModel) ssm).getPreviouslySelectedObject();
				repaint(obj);
			}

			// Repaint new selection.
			if (sse.getChange() != null)
			{
				repaint(sse.getChange());
			}
		}

		private void repaint(Object o)
		{
			Rectangle rect;

			if (o instanceof Switch)
			{
				CubicCurve2D[] cc = transformSwitchToView((Switch) o);
				rect = cc[0].getBounds();
				rect.add(cc[1].getBounds());
				rect.grow(4, 4);
				if (sce.getScale() == 1.0)
					sce.repaint( sce.getVisibleRect() );
				else
					sce.repaint(rect);
			}
			else if (o instanceof Track)
			{
				CubicCurve2D cc = transformTrackToView((Track) o);
				rect = cc.getBounds();
				rect.grow(4, 4);
				if (sce.getScale() == 1.0)
					sce.repaint( sce.getVisibleRect() );
				else
					sce.repaint(rect);
			}			
		}
	}

	/**
	 * Zooming in action.
	 */
	public class ZoomInAction extends AbstractAction
	{
		private JSceneryEditor editor;

		public ZoomInAction(JSceneryEditor s)
		{
			editor = s;
		}

		public void actionPerformed(ActionEvent ae)
		{
			double scale = sce.getScale();
			scale = incScale(scale);
			//sce.setScale(scale);

			Rectangle rect = sce.getVisibleRect();
			Point p = new Point(rect.x, rect.y);
			p.x += rect.width / 2;
			p.y += rect.height / 2;
			Point3d p3d = sce.viewToModel(p);

			sce.setScale( scale );

			sce.centerViewAt( sce.modelToView(p3d) );
		}
	}

	/**
	 * Zooming out action.
	 */
	public class ZoomOutAction extends AbstractAction
	{
		private JSceneryEditor editor;

		public ZoomOutAction(JSceneryEditor s)
		{
			editor = s;
		}

		public void actionPerformed(ActionEvent ae)
		{
			double scale = sce.getScale();
			scale = decScale(scale);
			//sce.setScale(scale);

			Rectangle rect = sce.getVisibleRect();
			Point p = new Point(rect.x, rect.y);
			p.x += rect.width / 2;
			p.y += rect.height / 2;
			Point3d p3d = sce.viewToModel(p);

			sce.setScale( scale );

			sce.centerViewAt( sce.modelToView(p3d) );
		}
	}

	/**
	 * Scenery selectio model with ability to remeber last selected object.
	 */
	class MemorableSelectionModel extends DefaultScenerySelectionModel
	{
		public MemorableSelectionModel(JSceneryEditor editor)
		{
			super(editor);
			oldSelectedValue = null;
		}

		public void clearSelection()
		{
			oldSelectedValue = super.getSelectedObject();
			super.clearSelection();
		}

		public void setSelectedObject(Object obj)
		{
			oldSelectedValue = getSelectedObject();
			super.setSelectedObject(obj);
		}

		public Object getPreviouslySelectedObject()
		{
			return oldSelectedValue;
		}

		private Object oldSelectedValue;
	}

	/*
	 * -------------------- Members -----------------
	 */

	/** Component. */
	protected JSceneryEditor sce;

	/** Data model. */
	private Scenery scenery = null;

	/** Scenery change listener. */ 
	private SceneryChangeHandler sceneryListener = null;

	/** Mouse move listener. */
	private HighlighterHandler highlighterListener = null;

	/** Property change listener. */
	private PropertyChangeHandler propertyChangeListener = null;

	/** Key listener. */
	private KeyHandler keyListener = null;

	/** Scenery selection listener. */
	private SelectionHandler scenerySelectionListener = null;

	/** Editor kit. */
	private static EditorKit defaultKit = new EditorKit();

	/** Scenery selection model. */
	private ScenerySelectionModel selectionModel;

	/** Dimension. */
	protected Dimension preferredSize = null;

	/** Validation of preferred size. */
	private boolean validPreferredSize;

	/** Validation of min max info. */
	private boolean validMinMaxInfo;

	/** Track color. */
	private Color trackColor;

	/** Switch color. */
	private Color switchColor;

	/** Zoom cursor. */
	private Cursor zoomCursor = null;

	/** Track names color. */
	private Color trackNamesColor;

	/** Track names visibility. */
	private boolean trackNamesVisibility;

	/** Red handler image. */
	private Image imageRedHandler;

	/** Base point image. */
	private Image imageBasePoint;

	/** Mouse controller mode. */
	private int mouseMode;

	/** Old cursor. */
	private Cursor oldCursor;
}
