/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import java.text.DecimalFormat;

import org.jet.*;

/**
 * Model for right switch palette button.
 *
 * <p>Indexes are in range from 0 to <i>count</i>-1.</p>
 *
 * @author Adam Domaski
 * @version 0.1 18/06/2004
 */
public class LeftSwitchButtonModel extends PaletteButtonModel
{
	/**
	 * Creates button model for left switch.
	 *
	 * @param l Length of the switch.
	 * @param r Radius of the switch.
	 * @param s <em>Skos</em> of switch
	 */
	public LeftSwitchButtonModel(double l, double r, String s)
	{
		if (l <= 0.0 || r <= 0.0)
			throw new IllegalArgumentException("Length and radius must be positive.");

		length = l;
		radius = r;
		skos = s;

		resource = JetResources.getInstance();

		setSelectedHandler(0);
	}

	/**
	 * Gets handlers count in this model.
	 *
	 * @return Always returns 3.
	 */
	public int getHandlerCount()
	{
		return 3;
	}

	/**
	 * Returns point of handler with specified index. Returned point is in the icon coordinates.
	 *
	 * @param i Index of the handler.
	 * @return Point in icon coordinates.
	 */
	public Point handlerToPoint(int i)
	{
		if (i == 0)
			return point1;
		else if (i == 1)
			return point2;
		else if (i == 2)
			return point3;

		return null;
	}

	/**
	 * Returns index of handler with specified point. If there is no handler at this point, it returns -1.
	 *
	 * @param p Point in icon coordiantes.
	 * @return Index of handler or -1 when ther is no handler.
	 */
	public int pointToHandler(Point p)
	{
		Rectangle rect = new Rectangle(point1);
		rect.grow(3, 3);
		if (rect.contains(p))
			return 0;
		rect.setLocation(point2.x - 3, point2.y - 3);
		if (rect.contains(p))
			return 1;
		rect.setLocation(point3.x - 3, point3.y - 3);
		if (rect.contains(p))
			return 2;
		
		return -1;
	}

	/**
	 * Gets an icon of this model.
	 */
	public Icon getIcon()
	{
		return icon;
	}

	/**
	 * Gets tooltip text.
	 */
	public String getToolTipText()
	{
		StringBuffer sb = new StringBuffer();

		sb.append( resource.translate("TrackPalette.switch2") );
		sb.append(" l=");
		sb.append( formatter.format(length) );
		sb.append(" m r=");
		sb.append( formatter.format(radius) );
		sb.append(" m n=");
		sb.append( skos );

		return sb.toString();
	}

	/*
	 * ------------ constants ---------------
	 */

	private static final Icon icon = new ImageIcon("images/jt_lswitch.png");

	private static final Point point1 = new Point(1, 7);
	private static final Point point2 = new Point(23, 7);
	private static final Point point3 = new Point(21, 2);

	private JetResources resource;

	private double length;
	private double radius;
	private String skos;

	private static DecimalFormat formatter = new DecimalFormat("0.00");
}
