/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Point2D;
import java.util.*;
import javax.swing.*;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.table.*;
import javax.vecmath.*;

import org.jet.*;
import org.jet.sce.*;

/**
 * Track palette.
 *
 * @author Adam Domaski
 * @version 0.1 18/06/2004
 */
public class TrackPalette implements Palette
{
	public TrackPalette(Jet app)
	{
		resource = JetResources.getInstance();

		application = app;

		buttonMap = new HashMap();
		buttonGroup = new ButtonGroup();
		tableModel = new PaletteTableModel();

		createListener();
		createStandalone();

		// Load tracks from script file.
		ScriptLoader scriptLoader = ScriptLoader.getInstance();
		try
		{
			scriptLoader.load(this);
		}
		catch (Exception ex)
		{
			JOptionPane.showMessageDialog(
				app.getFrame(),
				resource.translate("TrackPalette.parsingError") + ": " + ex + ".",
				"jET",
				JOptionPane.ERROR_MESSAGE);
		}
	}

	private void createStandalone()
	{
		JToggleButton jToggle;

		JToggleButton jHiddenButton = new JToggleButton();
		jHiddenButton.setVisible(false);
		buttonGroup.add(jHiddenButton);

		// Prosta
		jToggle = new JToggleButton();
		jToggle.setText( resource.translate("TrackPalette.other") );
		jToggle.setToolTipText( resource.translate("TrackPalette.straight") );
		jToggle.setMargin(new Insets(3, 3, 3, 3));
		jToggle.addActionListener(new StraightAction());
		putButton(jToggle, 0, 0);
		buttonGroup.add(jToggle);

		// uk
		jToggle = new JToggleButton();
		jToggle.setText( resource.translate("TrackPalette.other") );
		jToggle.setToolTipText( resource.translate("TrackPalette.arc") );
		jToggle.setMargin(new Insets(3, 3, 3, 3));
		jToggle.addActionListener(new ArcAction());
		putButton(jToggle, 1, 0);
		buttonGroup.add(jToggle);

		// Rozjazd
		jToggle = new JToggleButton();
		jToggle.setText( resource.translate("TrackPalette.switch") );
		jToggle.setToolTipText( resource.translate("TrackPalette.switch2") );
		jToggle.setMargin(new Insets(3, 3, 3, 3));
		jToggle.addActionListener(new SwitchAction());
		putButton(jToggle, 2, 0);
		buttonGroup.add(jToggle);

		// Guma
		jToggle = new JToggleButton();
		jToggle.setText( resource.translate("TrackPalette.flex") );
		jToggle.setToolTipText( resource.translate("TrackPalette.flex2") );
		jToggle.setMargin(new Insets(3, 3, 3, 3));
		FlexHandler fHandler = new FlexHandler(jHiddenButton);
		jToggle.addItemListener(fHandler);
		jToggle.addActionListener(fHandler);
		putButton(jToggle, 2, 1);
		buttonGroup.add(jToggle);
	}

	private void createListener()
	{
		trackButtonListener = new TrackButtonListener();
	}

	/*
	 * ------------ Buttons --------------
	 */
	
	/**
	 * Puts a toggle button to palette table at point (x, y).
	 */
	public void putButton(JToggleButton button, int x, int y)
	{
		Point p = new Point(x, y);
		buttonMap.put(p, button);
	}

	/**
	 * Puts a toggle button to palette table at specified column.
	 *
	 * @param button A button to add.
	 * @param x Column.
	 */
	public void putButton(JToggleButton button, int x)
	{
		int y = getRowCount(x);
		putButton(button, x, y);
	}

	/**
	 * Returns row count at specified column.
	 */
	private int getRowCount(int x)
	{
		int max = 0;

		Iterator i = buttonMap.keySet().iterator();
		while (i.hasNext())
		{
			Point p = (Point) i.next();
			if (p.x == x)
				max = Math.max(max, p.y);
		}

		return max+1;
	}

	/**
	 * Gets toggle button pointed to (x, y).
	 */
	public JToggleButton getButton(int x, int y)
	{
		Point p = new Point(x, y);
		if (!buttonMap.containsKey(p))
			return new BlankButton();
		return (JToggleButton) buttonMap.get(p);
	}

	/**
	 * Lays out a track button with palette model on the palette.
	 *
	 * @param b A button to layout.
	 */
	public void layout(JPaletteButton b)
	{
		PaletteButtonModel model = b.getPaletteModel();

		if (model instanceof StraightTrackButtonModel)
		{
			putButton(b, 0);
		}
		else if (model instanceof ArcTrackButtonModel ||
			model instanceof FlexTrackButtonModel)
		{
			putButton(b, 1);
		}
		else
		{
			putButton(b, 2);
		}
		buttonGroup.add(b);
		b.addActionListener(trackButtonListener);
	}

	/*
	 * ---------- Palette interface ----------
	 */

	/**
	 * Gets palette name.
	 */
	public String getName()
	{
		return resource.translate( "TrackPalette.name" );
	}

	/**
	 * Gets table model for palette table component.
	 */
	public TableModel getTableModel()
	{
		return tableModel;
	}

	/*
	 * --------- Inner classes ------------
	 */
	
	/**
	 * Palette table model for track palette.
	 */
	public class PaletteTableModel extends AbstractTableModel
	{
		/**
		 * Returns row count.
		 */
		public int getRowCount()
		{
			int max = 0;

			Iterator i = buttonMap.keySet().iterator();
			while (i.hasNext())
			{
				Point p = (Point) i.next();
				max = Math.max(max, p.y);
			}

			return max+1;
		}

		/**
		 * Table has 3 columns.
		 */
		public int getColumnCount()
		{
			return 3;
		}

		/**
		 * Gets button pointed by (col, row).
		 */
		public Object getValueAt(int row, int col)
		{
			return getButton(col, row);
		}

		/**
		 * Only enabled buttons are editable.
		 */
		public boolean isCellEditable(int row, int col)
		{
			JToggleButton b = getButton(col, row);
			if (!b.isEnabled())
				return false;

			return true;
		}

		/**
		 * Returns <code>JToggleButton.class</code>.
		 */
		public Class getColumnClass(int col)
		{
			return JToggleButton.class;
		}
	}

	/*
	 * --------- Handlers ----------
	 */
	
	class StraightAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			new CustomStraightProperties(application);
		}
	}

	class ArcAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			new CustomArcProperties(application);
		}
	}

	class SwitchAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			new CustomSwitchProperties(application);
		}
	}

	class FlexHandler extends InternalFrameAdapter
		implements ItemListener, ContainerListener, ActionListener
	{
		public FlexHandler(JToggleButton btn)
		{
			this.btn = btn;
			mouseHandler = new FlexMouseHandler();
		}

		public void itemStateChanged(ItemEvent ie)
		{
			int state = ie.getStateChange();
			if (state == ItemEvent.SELECTED)
			{
				application.getDesktopPane().addContainerListener(this);

				// Ustaw dla wszystkich okien edytorw nasuch
				Iterator itr = application.getEditorFrames().iterator();
				while (itr.hasNext())
				{
					JInternalFrame jFrame = (JInternalFrame) itr.next();
					jFrame.addInternalFrameListener(this);
				}

				SceneryFrame sceneryFrame = application.getTopmostFrame();
				if (sceneryFrame != null)
				{
					JSceneryEditor jEditor = sceneryFrame.getSceneryEditor();
					jEditor.addMouseListener(mouseHandler);
				}
			}
			else if (state == ItemEvent.DESELECTED)
			{
				application.getDesktopPane().removeContainerListener(this);

				// Ustaw dla wszystkich okien edytorw nasuch
				Iterator itr = application.getEditorFrames().iterator();
				while (itr.hasNext())
				{
					JInternalFrame jFrame = (JInternalFrame) itr.next();
					jFrame.removeInternalFrameListener(this);
				}

				SceneryFrame sceneryFrame = application.getTopmostFrame();
				if (sceneryFrame != null)
				{
					JSceneryEditor jEditor = sceneryFrame.getSceneryEditor();
					jEditor.removeMouseListener(mouseHandler);
				}
			}
		}

		public void internalFrameActivated(InternalFrameEvent ife)
		{
			JInternalFrame jFrame = ife.getInternalFrame();
			if (jFrame instanceof SceneryFrame)
			{
				JSceneryEditor jEditor = ((SceneryFrame) jFrame).getSceneryEditor();
				jEditor.addMouseListener(mouseHandler);
			}
		}

		public void internalFrameDeactivated(InternalFrameEvent ife)
		{
			JInternalFrame jFrame = ife.getInternalFrame();
			if (jFrame instanceof SceneryFrame)
			{
				JSceneryEditor jEditor = ((SceneryFrame) jFrame).getSceneryEditor();
				jEditor.removeMouseListener(mouseHandler);
			}
		}

		public void componentAdded(ContainerEvent ce)
		{
			Component aComp = ce.getChild();
			if (aComp instanceof JInternalFrame)
			{
				JInternalFrame jFrame = (JInternalFrame) aComp;
				jFrame.addInternalFrameListener(this);
			}
		}

		public void componentRemoved(ContainerEvent ce)
		{
			Component aComp = ce.getChild();
			if (aComp instanceof JInternalFrame)
			{
				JInternalFrame jFrame = (JInternalFrame) aComp;
				jFrame.removeInternalFrameListener(this);
			}
		}

		public void actionPerformed(ActionEvent ae)
		{
			Component aComp = (Component) ae.getSource();
			if (aComp instanceof JToggleButton)
			{
				JToggleButton jFlex = (JToggleButton) aComp;

				if (!selected)
					selected = true;
				else if (jFlex.isSelected())
				{
					btn.setSelected(true);
					selected = false;
				}
			}
		}

		private FlexMouseHandler mouseHandler = null;
		private JToggleButton btn;
		private boolean selected = false;
	}

	/**
	 * Mouse listener that listens to the topmost editor frame when
	 * flex button is selected. It adds to the editor scenery
	 * flex track.
	 */
	/*
	 * 1) Sprawd czy jest wogle jakiekolwiek zaznaczenie w edytorze.
	 * 2) Jeeli jest, to sprawd czy trafilimy w uchwyt.
	 * 3) Jeeli tak, to zapamitaj punkt i wektor.
	 * 4) Jeeli zosta zapamitany wczeniej punkt i wektor
	 */
	private class FlexMouseHandler extends MouseAdapter
	{
		public void mouseClicked(MouseEvent me)
		{
			Point targetPoint = me.getPoint();
			Object obj = me.getSource();

			if (obj instanceof JSceneryEditor)
			{
				JSceneryEditor jEditor = (JSceneryEditor) obj;

				// Sprawd zaznaczenie
				if (!jEditor.isSelectionEmpty())
				{
					Object selectedObject = jEditor.getSelectedObject();
					if (selectedObject instanceof Switch)
					{
						Switch sswitch = (Switch) selectedObject;

						// sprawd czy trafilimy w uchwyt
						if (checkPoint( jEditor.modelToView(sswitch.getPoint1()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, sswitch.getPoint1(), sswitch.getVector1());
							else
								compute(jEditor, selectedObject, sswitch.getPoint1(), sswitch.getVector1());
						}
						else if (checkPoint( jEditor.modelToView(sswitch.getPoint2()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, sswitch.getPoint2(), sswitch.getVector2());
							else
								compute(jEditor, selectedObject, sswitch.getPoint2(), sswitch.getVector2());
						}
						else if (checkPoint(  jEditor.modelToView(sswitch.getPoint3()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, sswitch.getPoint3(), sswitch.getVector3());
							else
								compute(jEditor, selectedObject, sswitch.getPoint3(), sswitch.getVector3());
						}
						else
						{
							//clear();
						}
					}
					else if (selectedObject instanceof Track)
					{
						Track track = (Track) selectedObject;

						if (checkPoint( jEditor.modelToView(track.getPoint1()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, track.getPoint1(), track.getVector1());
							else
								compute(jEditor, selectedObject, track.getPoint1(), track.getVector1());
						}
						else if (checkPoint( jEditor.modelToView(track.getPoint2()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, track.getPoint2(), track.getVector2());
							else
								compute(jEditor, selectedObject, track.getPoint2(), track.getVector2());
						}
						else
						{
							//clear();
						}
					}
				}
				else
				{
					//clear();
				}
			}
		}

		private boolean checkPoint(Point2D p, Point target)
		{
			Rectangle rect = new Rectangle((int) p.getX(), (int) p.getY(), 0, 0);
			rect.grow(4, 4);
			return rect.contains(target);
		}

		private void setOld(Object selection, Point3d p, Vector3d v)
		{
			oldSelection = selection;
			oldPoint = p;
			oldVector = v;
		}

		// Nie ma jeszcze midzy zwrotkami
		private void compute(JSceneryEditor jEditor, Object selection, Point3d p, Vector3d v)
		{
			if (selection != oldSelection)
			{
				Track track = new Track();

				double l;
				double dst = p.distance(oldPoint) / 3;

				Vector3d v1 = new Vector3d(oldVector);
				v1.negate();
				if (v1.x == -0.0)
					v1.x = 0.0;
				if (v1.y == -0.0)
					v1.y = 0.0;
				if (v1.z == -0.0)
					v1.z = 0.0;

				if (oldSelection instanceof Switch)
				{
					Switch sswitch = (Switch) oldSelection;
					Point3d pTemp;

					if (sswitch.getPoint1().equals(oldPoint) && sswitch.isStraight())
					{
						pTemp = sswitch.getPoint2();

						v1.x = oldPoint.x - pTemp.x;
						v1.y = oldPoint.y - pTemp.y;
						v1.z = oldPoint.z - pTemp.z;
					}
					else if (sswitch.getPoint2().equals(oldPoint) && sswitch.isStraight())
					{
						pTemp = sswitch.getPoint1();

						v1.x = oldPoint.x - pTemp.x;
						v1.y = oldPoint.y - pTemp.y;
						v1.z = oldPoint.z - pTemp.z;
					}
				}
				else if (oldSelection instanceof Track)
				{
					Track strTrack = (Track) oldSelection;
					if (strTrack.isStraight())
					{
						Point3d pTemp;

						if (strTrack.getPoint1().equals(oldPoint))
						{
							pTemp = strTrack.getPoint2();
						}
						else
						{
							pTemp = strTrack.getPoint1();
						}

						v1.x = 0.1 * (oldPoint.x - pTemp.x);
						v1.y = 0.1 * (oldPoint.y - pTemp.y);
						v1.z = 0.1 * (oldPoint.z - pTemp.z);						
					}
				}

				// Normalize
				l = v1.length();
				v1.scale(dst / l);

				Vector3d v2 = new Vector3d(v);
				v2.negate();
				if (v2.x == -0.0)
					v2.x = 0.0;
				if (v2.y == -0.0)
					v2.y = 0.0;
				if (v2.z == -0.0)
					v2.z = 0.0;

				if (selection instanceof Switch)
				{
					Switch sswitch = (Switch) selection;
					Point3d pTemp;

					if (sswitch.getPoint1().equals(p) && sswitch.isStraight())
					{
						pTemp = sswitch.getPoint2();

						v2.x = p.x - pTemp.x;
						v2.y = p.y - pTemp.y;
						v2.z = p.z - pTemp.z;
					}
					else if (sswitch.getPoint2().equals(p) && sswitch.isStraight())
					{
						pTemp = sswitch.getPoint1();

						v2.x = p.x - pTemp.x;
						v2.y = p.y - pTemp.y;
						v2.z = p.z - pTemp.z;
					}
				}
				else if (selection instanceof Track)
				{
					Track strTrack = (Track) selection;
					if (strTrack.isStraight())
					{
						Point3d pTemp;

						if (strTrack.getPoint1().equals(p))
						{
							pTemp = strTrack.getPoint2();
						}
						else
						{
							pTemp = strTrack.getPoint1();
						}

						v2.x = 0.1 * (p.x - pTemp.x);
						v2.y = 0.1 * (p.y - pTemp.y);
						v2.z = 0.1 * (p.z - pTemp.z);						
					}
				}

				// Normalize
				l = v2.length();
				v2.scale(dst / l);

				track.setPoint1( new Point3d(oldPoint) );
				track.setPoint2( new Point3d(p) );

				track.setVector1( v1 );
				track.setVector2( v2 );

				track.setLength(0.0);
				track.setRadius(0.0);
				track.setWidth(1.435);
				track.setTap(20.0);
				track.setFriction(0.25);
				track.setQuality(20);
				track.setDamage(0);
				track.setEnviroment( Track.FLAT );
				track.setVisible(true);
				track.setTexture( Track.TEXTURE_DEFAULT );
				track.setTextureLength(4.0);
				track.setSecondTexture( Track.SECOND_TEXTURE_DEFAULT );
				track.setTextureWidth(0.5);
				track.setTextureHeight(0.2);
				track.setTextureSlope(1.1);
				track.setRoll1(0.0);
				track.setRoll2(0.0);

				Scenery scenery = jEditor.getScenery();

				Object[] aObj = new Object[1];
				aObj[0] = track;
				scenery.insert(aObj);

				jEditor.setSelectedObject(track);

				jEditor.setBasePoint(null);
			}

			clear();
		}

		public void clear()
		{
			oldPoint = null;
			oldVector = null;
			oldSelection = null;
		}

		private boolean isOld()
		{
			return (oldPoint != null && oldVector != null);
		}

		private Point3d oldPoint = null;
		private Vector3d oldVector = null;
		private Object oldSelection = null;
	}

	/**
	 * Listener for track buttons.
	 */
	class TrackButtonListener implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			if (ae.getSource() instanceof JPaletteButton)
			{
				JPaletteButton b = (JPaletteButton) ae.getSource();
				PaletteButtonModel model = b.getPaletteModel();

				if (model.isSelectionEmpty())
					return;

				if (model instanceof StraightTrackButtonModel)
				{
					processStraight(model, (Track) b.getValue());
				}
				else if (model instanceof ArcTrackButtonModel ||
					model instanceof FlexTrackButtonModel)
				{
					processArc(model, (Track) b.getValue());
				}
				else if (model instanceof LeftSwitchButtonModel ||
					model instanceof RightSwitchButtonModel)
				{
					processSwitch(model, (Switch) b.getValue());
				}
			}
		}

		private void processStraight(PaletteButtonModel model, Track t)
		{
			Track track = (Track) t.clone();

			// Wsad to teraz do scenerii
			SceneryFrame frame = application.getTopmostFrame();
			if (frame != null)
			{
				JSceneryEditor editor = frame.getSceneryEditor();

				BasePoint basePoint = editor.getBasePoint();
				
				if (basePoint != null)
				{
				// Wykonaj obrt toru
				double alpha = - basePoint.getAlpha();
				double beta = - basePoint.getBeta();

				Matrix4d rotMatrix = new Matrix4d();
				rotMatrix.rotY(alpha);

				rotMatrix.transform( track.getPoint1() );
				rotMatrix.transform( track.getVector1() );
				rotMatrix.transform( track.getVector2() );
				rotMatrix.transform( track.getPoint2() );

				AxisAngle4d rotAxis = 
					new AxisAngle4d(track.getPoint2().z, 0.0, -track.getPoint2().x, beta);
				rotMatrix = new Matrix4d();
				rotMatrix.set(rotAxis);

				rotMatrix.transform( track.getPoint1() );
				rotMatrix.transform( track.getVector1() );
				rotMatrix.transform( track.getVector2() );
				rotMatrix.transform( track.getPoint2() );

				// wykonaj przesunicie toru
				Matrix4d traMatrix = new Matrix4d();
				traMatrix.setIdentity();
				traMatrix.setTranslation( new Vector3d(basePoint.getPoint()) );

				traMatrix.transform( track.getPoint1() );
				traMatrix.transform( track.getVector1() );
				traMatrix.transform( track.getVector2() );
				traMatrix.transform( track.getPoint2() );

				Scenery scenery = editor.getScenery();
				Object[] obj = new Object[1];
				obj[0] = track;
				scenery.insert(obj);

				// Zaznacz nowy obiekt
				editor.setSelectedObject(track);

				// Ustaw nowy punkt bazowy
				Point3d point = track.getPoint2();
				Vector3d resolved = null;

				Point3d p1 = track.getPoint1();
				resolved = new Vector3d();
				resolved.x = point.x - p1.x;
				resolved.y = point.y - p1.y;
				resolved.z = point.z - p1.z;

				BasePoint nBasePoint = new BasePoint(point, resolved);
				editor.setBasePoint(nBasePoint);

				editor.centerViewAt( editor.modelToView(point) );
				}
			}
		}

		private void processArc(PaletteButtonModel model, Track t)
		{
			Track track = (Track) t.clone();

			// Sprawd, ktry handler jest zaznaczony
			int index = model.getSelectedHandler();
			if (index == 1)
			{
				double ang;
				Vector3d v = track.getVector2();
				Point3d p;

				// Przesu ukad wsprzdnych w drugi punkt toru
				Matrix4d mTranslate = new Matrix4d();
				mTranslate.setIdentity();
				Vector3d vTranslate = new Vector3d(track.getPoint2());
				vTranslate.negate();
				mTranslate.setTranslation( vTranslate );
				ang = Math.atan2(-v.x, v.z);

				mTranslate.transform( track.getPoint1() );
				mTranslate.transform( track.getVector1() );
				mTranslate.transform( track.getVector2() );
				mTranslate.transform( track.getPoint2() );

				// Obr
				mTranslate.setZero();
				mTranslate.rotY(ang);

				mTranslate.transform( track.getPoint1() );
				mTranslate.transform( track.getVector1() );
				mTranslate.transform( track.getVector2() );
				mTranslate.transform( track.getPoint2() );

				// Zamie wsprzdne
				p = track.getPoint1();
				track.setPoint1( track.getPoint2() );
				track.setPoint2( p );
				v = track.getVector1();
				track.setVector1( track.getVector2() );
				track.setVector2( v );
			}

			// Wsad to teraz do scenerii
			SceneryFrame frame = application.getTopmostFrame();
			if (frame != null)
			{
				JSceneryEditor editor = frame.getSceneryEditor();

				BasePoint basePoint = editor.getBasePoint();
				
				if (basePoint != null)
				{
				// Wykonaj obrt toru
				double alpha = - basePoint.getAlpha();
				double beta = - basePoint.getBeta();

				Matrix4d rotMatrix = new Matrix4d();
				rotMatrix.rotY(alpha);

				rotMatrix.transform( track.getPoint1() );
				rotMatrix.transform( track.getVector1() );
				rotMatrix.transform( track.getVector2() );
				rotMatrix.transform( track.getPoint2() );

				AxisAngle4d rotAxis = 
					new AxisAngle4d(track.getPoint2().z, 0.0, -track.getPoint2().x, beta);
				rotMatrix = new Matrix4d();
				rotMatrix.set(rotAxis);

				rotMatrix.transform( track.getPoint1() );
				rotMatrix.transform( track.getVector1() );
				rotMatrix.transform( track.getVector2() );
				rotMatrix.transform( track.getPoint2() );

				// wykonaj przesunicie toru
				Matrix4d traMatrix = new Matrix4d();
				traMatrix.setIdentity();
				traMatrix.setTranslation( new Vector3d(basePoint.getPoint()) );

				traMatrix.transform( track.getPoint1() );
				traMatrix.transform( track.getVector1() );
				traMatrix.transform( track.getVector2() );
				traMatrix.transform( track.getPoint2() );

				Scenery scenery = editor.getScenery();
				Object[] obj = new Object[1];
				obj[0] = track;
				scenery.insert(obj);

				// Zaznacz nowy obiekt
				editor.setSelectedObject(track);

				// Ustaw nowy punkt bazowy
				Point3d point = track.getPoint2();
				Vector3d resolved = null;

				resolved = new Vector3d( track.getVector2() );
				resolved.negate();

				BasePoint nBasePoint = new BasePoint(point, resolved);
				editor.setBasePoint(nBasePoint);

				editor.centerViewAt( editor.modelToView(point) );
				}
			}
		}

		private void processSwitch(PaletteButtonModel model, Switch s)
		{
			Switch sswitch = (Switch) s.clone();

			// Sprawd, ktry handler jest zaznaczony
			int index = model.getSelectedHandler();
			if (index == 1)
			{
				double ang;
				Point3d p = sswitch.getPoint2();

				Matrix4d mTranslate = new Matrix4d();

				if (sswitch.isStraight())
				{
					ang = Math.PI;
				}
				else
				{
					Vector3d v = sswitch.getVector2();
					ang = Math.atan2(-v.x, v.z);
				}

				mTranslate.rotY(ang);
				mTranslate.setTranslation( new Vector3d(p) );

				mTranslate.transform( sswitch.getPoint1() );
				mTranslate.transform( sswitch.getPoint2() );
				mTranslate.transform( sswitch.getPoint3() );
				mTranslate.transform( sswitch.getVector1() );
				mTranslate.transform( sswitch.getVector2() );
				mTranslate.transform( sswitch.getVector3() );
				mTranslate.transform( sswitch.getSwitchedVector() );
			}
			else if (index == 2)
			{
				double ang;
				Point3d p = sswitch.getPoint3();
				Vector3d v = sswitch.getVector3();

				Matrix4d mTranslate = new Matrix4d();

				Vector3d trV = new Vector3d(p);
				trV.negate();

				ang = Math.atan2(-v.x, v.z);

				mTranslate.setIdentity();
				mTranslate.setTranslation(trV);

				mTranslate.transform( sswitch.getPoint1() );
				mTranslate.transform( sswitch.getPoint2() );
				mTranslate.transform( sswitch.getPoint3() );
				mTranslate.transform( sswitch.getVector1() );
				mTranslate.transform( sswitch.getVector2() );
				mTranslate.transform( sswitch.getVector3() );
				mTranslate.transform( sswitch.getSwitchedVector() );

				mTranslate.setZero();
				mTranslate.rotY(ang);

				mTranslate.transform( sswitch.getPoint1() );
				mTranslate.transform( sswitch.getPoint2() );
				mTranslate.transform( sswitch.getPoint3() );
				mTranslate.transform( sswitch.getVector1() );
				mTranslate.transform( sswitch.getVector2() );
				mTranslate.transform( sswitch.getVector3() );
				mTranslate.transform( sswitch.getSwitchedVector() );
			}

			// Wsad to teraz do scenerii
			SceneryFrame frame = application.getTopmostFrame();
			if (frame != null)
			{
				JSceneryEditor editor = frame.getSceneryEditor();

				BasePoint basePoint = editor.getBasePoint();
				
				if (basePoint != null)
				{
				// Wykonaj obrt toru
				double alpha = - basePoint.getAlpha();
				double beta = - basePoint.getBeta();

				Matrix4d rotMatrix = new Matrix4d();
				rotMatrix.rotY(alpha);

				rotMatrix.transform( sswitch.getPoint1() );
				rotMatrix.transform( sswitch.getVector1() );
				rotMatrix.transform( sswitch.getVector2() );
				rotMatrix.transform( sswitch.getPoint2() );
				rotMatrix.transform( sswitch.getPoint3() );
				rotMatrix.transform( sswitch.getVector3() );
				rotMatrix.transform( sswitch.getSwitchedVector() );

				AxisAngle4d rotAxis = 
					new AxisAngle4d(sswitch.getPoint2().z, 0.0, -sswitch.getPoint2().x, beta);
				rotMatrix = new Matrix4d();
				rotMatrix.set(rotAxis);

				rotMatrix.transform( sswitch.getPoint1() );
				rotMatrix.transform( sswitch.getVector1() );
				rotMatrix.transform( sswitch.getVector2() );
				rotMatrix.transform( sswitch.getPoint2() );
				rotMatrix.transform( sswitch.getPoint3() );
				rotMatrix.transform( sswitch.getVector3() );
				rotMatrix.transform( sswitch.getSwitchedVector() );

				// wykonaj przesunicie toru
				Matrix4d traMatrix = new Matrix4d();
				traMatrix.setIdentity();
				traMatrix.setTranslation( new Vector3d(basePoint.getPoint()) );

				traMatrix.transform( sswitch.getPoint1() );
				traMatrix.transform( sswitch.getVector1() );
				traMatrix.transform( sswitch.getVector2() );
				traMatrix.transform( sswitch.getPoint2() );
				traMatrix.transform( sswitch.getPoint3() );
				traMatrix.transform( sswitch.getVector3() );
				traMatrix.transform( sswitch.getSwitchedVector() );

				Scenery scenery = editor.getScenery();
				Object[] obj = new Object[1];
				obj[0] = sswitch;
				scenery.insert(obj);

				// Zaznacz nowy obiekt
				editor.setSelectedObject(sswitch);

				// Ustaw nowy punkt bazowy
				Point3d p1 = sswitch.getPoint1();
				Point3d p2 = sswitch.getPoint2();
				Point3d p3 = sswitch.getPoint3();
				Point3d newPoint;
				Vector3d resolved = null;

				if (basePoint.getPoint().epsilonEquals(p1, 0.001))
					newPoint = new Point3d(p2);
				else 
					newPoint = new Point3d(p1);
				
				if (newPoint.equals(p1))
				{
					if (!sswitch.isStraight())
					{
						resolved = new Vector3d( sswitch.getVector1() );
						resolved.negate();
					}
					else
					{
						Point3d p = p2;
						resolved = new Vector3d();
						resolved.x = newPoint.x - p.x;
						resolved.y = newPoint.y - p.y;
						resolved.z = newPoint.z - p.z;
					}
				}
				else if (newPoint.equals(p2))
				{
					if (!sswitch.isStraight())
					{
						resolved = new Vector3d( sswitch.getVector2() );
						resolved.negate();
					}
					else
					{
						Point3d p = p1;
						resolved = new Vector3d();
						resolved.x = newPoint.x - p.x;
						resolved.y = newPoint.y - p.y;
						resolved.z = newPoint.z - p.z;
					}
				}
				else if (newPoint.equals(p3))
				{
					resolved = new Vector3d( sswitch.getVector3() );
					resolved.negate();
				}
					
				BasePoint newBasePoint = new BasePoint(newPoint, resolved);
				editor.setBasePoint(newBasePoint);

				editor.centerViewAt( editor.modelToView(newPoint) );
				}
			}
		}
	}

	/**
	 * Blank inactive toggle button.
	 */
	class BlankButton extends JToggleButton
	{
		public BlankButton()
		{
			super();
			setEnabled(false);
		}
	}

	/*
	 * --------- Members ----------
	 */
	
	/** I18n resources. */
	private JetResources resource;

	/** Map containing buttons */
	private Map buttonMap;

	/** Button group. */
	private ButtonGroup buttonGroup;

	/** Table model. */
	private PaletteTableModel tableModel;

	/** Application. */
	private Jet application;

	/** Listener for track buttons. */
	private TrackButtonListener trackButtonListener;
}
