/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package prv.doman.util;

import java.util.Locale;
import java.text.DecimalFormatSymbols;

/**
 * Defines special symbols for degree format under specified locale.
 *
 * @author Adam Domaski
 * @version 0.1 23/07/2003
 */
public class DegreeFormatSymbols
{
	/**
	 * Creates instance of degree format symbols with default locale.
	 */
	public DegreeFormatSymbols()
	{
		this(Locale.getDefault());
	}

	/**
	 * Creates instance of degree format symbols with specified locale.
	 *
	 * @param locale Locale.
	 */
	public DegreeFormatSymbols(Locale locale)
	{
		decimalSymbols = new DecimalFormatSymbols(locale);
	}

	// -------------------- Decimal ---------------------

	/**
	 * Sets decimal separator. It divides a number to integer and fractional parts.
	 *
	 * @param ch Character to be used as decimal separator.
	 */
	public void setDecimalSeparator(char ch)
	{
		decimalSymbols.setDecimalSeparator(ch);
	}

	/**
	 * Gets decimal separator. It divides a number to integer and fractional parts.
	 *
	 * @return Character used as decimal separator.
	 */
	public char getDecimalSeparator()
	{
		return decimalSymbols.getDecimalSeparator();
	}

	// ---------------- Degree ------------------

	/**
	 * Sets degree symbol.
	 *
	 * @param ch Character to be used as degree symbol.
	 */
	public void setDegree(char ch)
	{
		degreeSymbol = ch;
	}

	/**
	 * Gets degree symbol.
	 * <p>This method is not locale dependent. The default value is Unicode
	 * symbol Degree Sign <code>&#xB0;</code> (&#x5C;u00B0).</p>
	 *
	 * @return Character for degree.
	 */
	public char getDegree()
	{
		return degreeSymbol;
	}

	/**
	 * Sets minute symbol.
     *
	 * @param ch Character to be used as minute symbol.
	 */
	public void setMinute(char ch)
	{
		minuteSymbol = ch;
	}

	/**
	 * Gets minute symbol.
	 * <p>This method is not locale dependent. The default symbol for this is Unicode
	 * Prime <code>&#x2032;</code> (&#x5C;u2032).</p>
	 *
	 * @return Character used as minute symbol.
	 */
	public char getMinute()
	{
		return minuteSymbol;
	}

	/**
	 * Sets the second symbol.
	 *
	 * @param ch Character to be used for second symbol.
	 */
	public void setSecond(char ch)
	{
		secondSymbol = ch;
	}

	/**
	 * Gets the second symbol.
	 * <p>The method is not locale dependent. The default value for this is Unicode
	 * Double Prime symbol <code>&#x2033;</code> (&#x5C;u2033).</p>
	 *
	 * @return Character used as second symbol.
	 */
	public char getSecond()
	{
		return secondSymbol;
	}

	// ---------- Object methods -----------

	/**
	 * Checks if symbols matches together.
	 */
	public boolean equals(Object obj)
	{
		if (obj == null)
			return false;
		if (obj == this)
			return true;
		if (obj.getClass() != getClass())
			return false;
		DegreeFormatSymbols other = (DegreeFormatSymbols) obj;
		return (decimalSymbols.equals(other) && 
			secondSymbol == other.secondSymbol &&
			minuteSymbol == other.minuteSymbol &&
			degreeSymbol == other.degreeSymbol
		);
	}

	/**
	 * Rehashes object.
	 */
	public int hashCode()
	{
		int result = decimalSymbols.hashCode();
		result = result * 37 + degreeSymbol;
		result = result * 37 + minuteSymbol;
		return result;
	}

	// ---------------- Privates -------------------

	private DecimalFormatSymbols decimalSymbols;
	private char degreeSymbol = '\u00B0';
	private char minuteSymbol = '\u2032';
	private char secondSymbol = '\u2033';
}
