/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package prv.doman.util;

import java.text.Format;
import java.text.FieldPosition;
import java.text.ParsePosition;
import java.text.ParseException;

/**
 * This class enables format and parse hexadecimal numbers.
 * It works with Java standard types of numbers: byte, short, int,
 * and long. Floating point types are discarded.
 * <p>Hexadacimal format is not locale dependent.</p>
 *
 * @author Adam Domaski
 * @version 0.1 10/07/2003
 */
public class HexadecimalFormat extends Format
{
	/**
	 * Decoration means additional format of hexadacimal format.
	 * This constant means no decoration.
	 */
	public static final int DECORATION_NONE = 0;
	/**
	 * Sets at the begining of output symbols: <code>0x</code>.
	 */
	public static final int DECORATION_PREFIX = 1;
	/**
	 * Sets at the end of output symbol: <code>h</code>.
	 */
	public static final int DECORATION_SUFFIX = 2;

	/**
	 * Creates default hexadecimal formatter.
	 * It uses symbols: <code>0</code> - <code>9</code> and <code>a</code>
	 * - <code>f</code>. It defines no decoration and lower case.
	 */
	public HexadecimalFormat()
	{
	}

	/**
	 * See note on superclass.
	 */
	public StringBuffer format(Object obj, StringBuffer toAppendTo, FieldPosition pos)
	{
		if (!((obj instanceof Byte) || (obj instanceof Short) ||
			(obj instanceof Integer) || (obj instanceof Long)))
			throw new IllegalArgumentException("Not integer number");

		if (toAppendTo == null)
			throw new NullPointerException("String buffer is null");

		if (decoration == DECORATION_PREFIX)
			toAppendTo.append("0x");

		if (obj instanceof Byte)
		{
			format(((Byte) obj).byteValue(), toAppendTo, pos);
		}
		else if (obj instanceof Short)
		{
			format(((Short) obj).shortValue(), toAppendTo, pos);
		}
		else if (obj instanceof Integer)
		{
			format(((Integer) obj).intValue(), toAppendTo, pos);
		}
		else if (obj instanceof Long)
		{
			format(((Long) obj).longValue(), toAppendTo, pos);
		}

		if (decoration == DECORATION_SUFFIX)
			toAppendTo.append('h');
		
		return toAppendTo;
	}

	/**
	 * Format single byte.
	 * Result will be two hexadecimal digits.
	 *
	 * @param b Byte value.
	 * @return Hexadecimal textual representation of this byte.
	 */
	final public String format(byte b)
	{
		return format(b, new StringBuffer(), new FieldPosition(0)).toString();
	}

	/**
	 * Formats single byte to the hexadecimal textual representation.
	 * The result appends to string buffer.
	 *
	 * @param b Byte value.
	 * @param toAppendTo String buffer where append to.
	 * @param pos Unused.
	 */
	public StringBuffer format(byte b, StringBuffer toAppendTo, FieldPosition pos)
	{
		return toAppendTo.append( divideAndConquer(b, 8) );
	}

	/**
	 * Formats short value to the hexadecimal textual representation.
	 *
	 * @param s Short value.
	 * @return Four hexadecimal digits.
	 */
	final public String format(short s)
	{
		return format(s, new StringBuffer(), new FieldPosition(0)).toString();
	}

	/**
	 * Formats short value to the hexidecimal textual representation.
	 *
	 * @param s Short value.
	 * @param toAppendTo Where text should be appended.
	 * @param pos Unused.
	 * @return String buffer with appended hexadecimal number.
	 */
	public StringBuffer format(short s, StringBuffer toAppendTo, FieldPosition pos)
	{
		return toAppendTo.append( divideAndConquer(s, 16) );
	}

	/**
	 * Formats integer value to the hexadecimal representation.
	 *
	 * @param i Integer value.
	 * @return Eight hexadecimal digits.
	 */
	final public String format(int i)
	{
		return format(i, new StringBuffer(), new FieldPosition(0)).toString();
	}

	/**
	 * Formats integer value to the hexidecimal textual representation.
	 *
	 * @param i Integer value.
	 * @param toAppendTo Where text should be appended.
	 * @param pos Unused.
	 * @return String buffer with appended hexadecimal number.
	 */
	public StringBuffer format(int i, StringBuffer toAppendTo, FieldPosition pos)
	{
		return toAppendTo.append( divideAndConquer(i, 32) );
	}

	/**
	 * Formats long value to the hexadecimal representation.
	 *
	 * @param l Long value.
	 * @return Sixteen hexadecimal digits.
	 */
	final public String format(long l)
	{
		return format(l, new StringBuffer(), new FieldPosition(0)).toString();
	}

	/**
	 * Formats long value to the hexidecimal textual representation.
	 *
	 * @param l Long value.
	 * @param toAppendTo Where text should be appended.
	 * @param pos Unused.
	 * @return String buffer with appended hexadecimal number.
	 */
	public StringBuffer format(long l, StringBuffer toAppendTo, FieldPosition pos)
	{
		return toAppendTo.append( divideAndConquer(l, 64) );
	}

	/**
	 * Parses text to find hexadecimal numbers.
	 * Returned object are instances of java.lang.Number class.
	 *
	 * @param source Text to search.
	 * @param pos Starting position of searching.
	 * @return Returned number.
	 */
	final public Object parseObject(String source, ParsePosition pos)
	{
		return parse(source, pos);
	}

	/**
	 * Parses text at specified position.
	 *
	 * @param source Source of text.
	 * @param pos Position to start searching from.
	 * @return Byte, short, integer or long wrapper of hexadecimal
	 *   number in <i>source</i>.
	 */
	public Number parse(String source, ParsePosition pos)
	{
		int digit;
		int position = pos.getIndex();
		int oldPosition = position;
		int digitCount = 0;
		long longResult = 0;

		if (source.startsWith("0x"))
			position += 2;

		while (position < source.length() && digitCount < 16)
		{
			char ch = source.charAt(position);
			if ((digit = Character.digit(ch, 16)) != -1)
			{
				longResult <<= 4;
				longResult |= digit;
				digitCount++;
			}
			else
			{
				if (digitCount == 0)
				{
					pos.setErrorIndex(oldPosition);
					pos.setIndex(oldPosition);
					return null;
				}
				break;
			}
			position++;
		}

		pos.setIndex(position+1);

		if (digitCount <= 2)
		{
			return new Byte((byte) longResult);
		}
		else if (digitCount > 2 && digitCount <= 4)
		{
			return new Short((short) longResult);
		}
		else if (digitCount > 4 && digitCount <= 8)
		{
			return new Integer((int) longResult);
		}
		else if (digitCount > 8 && digitCount <= 16)
		{
			return new Long(longResult);
		}
		else
		{
			pos.setErrorIndex(oldPosition);
			pos.setIndex(oldPosition);
		}

		return null;
	}

	/**
	 * Parses text containing hexadecimal number. A number must have
	 * at most 16 digits. Otherwise parse exception is thrown.
	 * Hexadecimal representation may contains prefix <code>0x</code> or 
	 * suffix <code>h</code>.
	 * <p>Exception is thrown when number hasn't got proper digit count or
	 * has bad format, or text isn't a hexadecimal number.</p>
	 *
	 * @param source Text containing hexadecimal number.
	 * @return Byte, short, integer or long wrapper if text contains
	 *   number, or null when excpetion is thrown.
	 * @throws ParseException Exception.
	 */
	public Number parse(String source) throws ParseException
	{
		ParsePosition pos = new ParsePosition(0);
		Number num = parse(source, pos);
		if (pos.getIndex() == 0)
			throw new ParseException("Unparseable number: \"" + source + "\"",
                pos.getErrorIndex());

		return num;
	}

	/**
	 * This methods divide every four bits from l per 16.
	 * Output is returned in string buffer.
	 *
	 * @param l Number.
	 * @param q How many bits is in number.
	 */
	private StringBuffer divideAndConquer(long l, int q)
	{
		int pom;
		char c;
		StringBuffer sb = new StringBuffer();

		for (int i = q-4; i >= 0; i -= 4)
		{
			pom = (int) ((l >> i) & 0xf);
			c = Character.forDigit(pom, 16);
			if (upperCase)
				c = Character.toUpperCase(c);
			sb.append(c);
		}

		return sb;
	}

	// ---------- Format options --------------

	/**
	 * Sets decoration for this format.
	 * Decaration styles are constants. Note that decoration with suffix
	 * and decoration with prefix exclude each other.
	 *
	 * @param d Decoration styles as integer.
	 */
	public void setDecoration(int d)
	{
		if (d == (DECORATION_PREFIX | DECORATION_SUFFIX))
		{
			d = DECORATION_NONE;
		}
		decoration = d;
	}

	/**
	 * Gets decoration style of this format.
	 *
	 * @return Decoration style represented by integer constant.
	 */
	public int getDecoration()
	{
		return decoration;
	}

	/**
	 * Sets hexadecimal digits from <code>a</code> to <code>f</code> to be
	 * upper cased.
	 */
	public void toUpperCase()
	{
		upperCase = true;
	}

	/**
	 * Sets hexadecimal digits from <code>a</code> to <code>f</code> to be
	 * lower cased.
	 */
	public void toLowerCase()
	{
		upperCase = false;
	}

	/**
	 * Checks if hexadecimal digits from <code>a</code> to <code>f</code> are
	 * lower or upper cased in this format.
	 *
	 * @return True if digits are lower cased.
	 */
	public boolean isLowerCase()
	{
		return upperCase;
	}

	// -------------- Privates -----------------

	private boolean upperCase = false;
	private int decoration = DECORATION_NONE;
}
